# Methods added to this helper will be available to all templates in the application.

module ApplicationHelper
  include BootstrapContentHelper

  def sortable(column, title = nil)
    title ||= column.titleize
    direction = ( (column == sort_column || sort_column == "sort_value") && sort_direction == "asc") ? "desc" : "asc"
    link_to title, params.merge(sort: CGI.unescapeHTML(column), direction: direction)
  end

  # HTML escapes and formats message strings for HTML output. "\n" will be replaced by "<br>" def format_and_escape(message)
  def format_and_escape(message)
    message ||= ""
    lines = []
    message.each_line {|line| lines << html_escape(line)}
    lines.join("<br />")
  end

  #========================================================================================================
  # AJAX helpers used by RJS view templates and RHTML views

  def ajax_live_update_javascript_code(ajax_request_url, params = {}, js_params = {})
        #Code included in live update forms
    #Placed here instead of in a .js file due to need to substitute in certain constants and the URL, as well as to include certain safety tags for non-javascript browsers
    #Could still be done as a JS include in future with addition of hidden controls or page properties etc.
    html_lines = []
    html_lines << '<script type="text/javascript">'
    html_lines << '/* <![CDATA[ */'
    html_lines << ''
    html_lines << '//'
    html_lines << '// AJAX low level code'
    html_lines << '//'
    html_lines << ''
    html_lines << '//page global variables used by our live update function'
    html_lines << 'var asperaSecondsUntilAjaxTimeout=' + FASPEX_LIVE_UPDATE_TIMEOUT.to_s + ';'
    html_lines << 'var asperaLastOutstandingRequestTime = null; //this gets cleared by the returning javascript snippet'
    html_lines << 'var asperaIntervalID = null;'
    html_lines << 'var asperaXMLHttpRequest = null;'
    html_lines << ''

    #======================================================================================================================================

    html_lines << 'function aspera_live_update(new_path) { //request an update of stats, but only if we have received a response from the last ajax request (if any)'
    html_lines << ''
    html_lines << '  jQuery(".retry_icon_small").click(function() {'
    html_lines << '    if (asperaXMLHttpRequest) {'
    html_lines << '      asperaXMLHttpRequest.abort();'
    html_lines << '    }'
    html_lines << '  });'
    html_lines << ''
    html_lines << '  jQuery(".gr_pkg_dl_icon_small").click(function() {'
    html_lines << '    if (asperaXMLHttpRequest) {'
    html_lines << '      asperaXMLHttpRequest.abort();'
    html_lines << '    }'
    html_lines << '  });'
    html_lines << ''
    html_lines << '  var send_update = false; //default to not asking for an update'
    html_lines << '  '
    html_lines << '  if (asperaLastOutstandingRequestTime) {'
    html_lines << '    //a previous request has gone out and has not been returned, check to see if we have timed out, in which case we will send again'
    html_lines << '    var now = new Date();'
    html_lines << '    var secondsElapsedSinceRequest = (now.getTime() - asperaLastOutstandingRequestTime.getTime()) / 1000; //convert milliseconds to seconds'
    html_lines << '    if (secondsElapsedSinceRequest >= asperaSecondsUntilAjaxTimeout) {'
    html_lines << '      //we have timed out, perhaps the last request got lost? send again'
    html_lines << '      send_update = true;'
    html_lines << '      }'
    html_lines << '    }'
    html_lines << '  else {'
    html_lines << '    //this is our first request, or the previous request succeeded and asperaLastOutstandingRequestTime got cleared'
    html_lines << '    send_update = true;'
    html_lines << '    }'
    html_lines << '  '
    html_lines << '  //send the request if appropriate'
    html_lines << '  if (send_update) {'
    html_lines << '    asperaLastOutstandingRequestTime = new Date(); //save the current date and time to indicate a request went out'
    html_lines << '    // prepare the ajax request'
    html_lines << '    if (asperaXMLHttpRequest) {'
    html_lines << '      //if one already exists, kill it, clean up'
    html_lines << '      asperaXMLHttpRequest.onreadystatechange=function() {} //clear event handler'
    html_lines << '      asperaXMLHttpRequest.abort();'
    html_lines << '      asperaXMLHttpRequest = null; //discard it'
    html_lines << '    }'
    html_lines << '    //get a new object'

    html_lines << '   jsParams = new Hash();'
    params.each_pair do |key, value|
      html_lines << "   jsParams.set('#{escape_javascript key.to_s}', '#{value.to_s}');"
    end

    # for unescaped values (e.g. accessing the DOM)
    js_params.each_pair do |key, value|
      html_lines << %Q[if($("#{key.to_s}") != null) { ] # not required in jQuery
      html_lines << %Q{     jsParams.set('#{key.to_s}', #{value});}
      html_lines << "}"
    end

    html_lines << 'if(new_path){'
    html_lines << '  jsParams.set("new_path", new_path);'
    html_lines << '}'
    html_lines << remote_function(:url => ajax_request_url,
                                  :with => "jsParams.toQueryString()",
                                  :method => :post)
    html_lines << '    asperaXMLHttpRequest = JQXHR;'
    html_lines << '  }'
    html_lines << '}'

    #======================================================================================================================================

    html_lines << 'function aspera_cancel_update() { //kill the setInterval routine, if an ajax request goes out after user has clicked on a link to go to another page, Safari crashes'
    html_lines << ''
    html_lines << '  if (asperaXMLHttpRequest) {'
    html_lines << '    //if one already exists, kill it, clean up'
    html_lines << '    asperaXMLHttpRequest.onreadystatechange=function() {} //clear event handler'
    html_lines << '    asperaXMLHttpRequest.abort();'
    html_lines << '    asperaXMLHttpRequest = null; //discard it'
    html_lines << '    }'
    html_lines << ''
    html_lines << '  if (asperaIntervalID) {'
    html_lines << '    window.clearInterval(asperaIntervalID);'
    html_lines << '    }'
    html_lines << '  }'
    html_lines << ''
    html_lines << ''
    html_lines << ' function aspera_start_update(new_path) { '
    html_lines << '    //initialize the timed, repeating event'
    html_lines << '    asperaIntervalID = window.setInterval(function(){ aspera_live_update(new_path) }, ' + FASPEX_LIVE_UPDATE_INTERVAL.to_s + ' * 1000); //asperaIntervalID is a reference to the update timer, can be used to cancel it'
    html_lines << '  '
    html_lines << '    //make sure the timer is canceled when user clicks to leave page'
    html_lines << '    window.onunload = aspera_cancel_update;'
    html_lines << '}'
    html_lines << 'aspera_start_update();'
    html_lines << ''

    #======================================================================================================================================

    html_lines << '    //function to attach a function to window.onload safely, without screwing up already attached handlers'
    html_lines << '    function addLoadEvent(func) {'
    html_lines << '        var oldonload = window.onload;'
    html_lines << '        if (typeof window.onload != "function") {'
    html_lines << '            window.onload = func;'
    html_lines << '        } else {'
    html_lines << '            window.onload = function() {'
    html_lines << '                if (oldonload) {'
    html_lines << '                    oldonload();'
    html_lines << '                }'
    html_lines << '                func();'
    html_lines << '            }'
    html_lines << '        }'
    html_lines << '    };'
    html_lines << ''
    html_lines << '    //do the first live update as soon as we finish loading page'
    html_lines << '    addLoadEvent(aspera_live_update);'
    html_lines << ''
    html_lines << '/* ]]> */'
    html_lines << '</script>'
    html_lines << ''

    return html_lines.join("\n")
  end

  def ajax_replace_if_different(page_object, element_id, replacement_string)
    page_object.replace_html(element_id , replacement_string)
  end

  def ajax_replace_if_img_src_different(page_object, element_id, image_html_string)
    #replace an html element if the image src is different from what we want
    #(try to be smart about it so we don't replace unnecessarily)
    page_object<< "var do_the_update = false;" #initialize to default of not updating

    #extract the path of the image source, minus protocol, hostname, and port
    new_image_src = "" #initialize the value in case there's no src="..." in the html generated
    if image_html_string.match(/ src="([^"]+)" /) #if the replacement URL contains a src="..." statement (with leading and trailing space)
      new_image_src = $1 #save what is between the quotation marks, isolate the text of img src
      #we should now have a path of the form:  /images/download.gif?1158813730
      #but just to make sure we don't have protocol and host name in front, do another step
      if new_image_src.match(/[^\/]+\/\/[^\/]+(\/.*$)/) #if we found a nonslash // nonslash / pattern
        new_image_src = $1 #take the part without the protocol / hostname
      end
    end

    #feed in the replacement HTML and extracted image src to the javascript code
    page_object<< "var replacementHTML = '" + escape_javascript(image_html_string) + "';"
    page_object<< 'var new_img_src = "' + escape_javascript(new_image_src) + '";'

    #check image src of existing element
    page_object<< "var table_element = $('" + element_id + "');"
    page_object<< "if (table_element) {"

    page_object<< '    var existing_img_src = "";' #initialize to 'img src' not found
    page_object<< '    var img_src_pattern_matches = table_element.innerHTML.match(/src="([^"]+)"/);' #use regexp to catch img src
    page_object<< '    if (img_src_pattern_matches) {' #if we found at least one match
    page_object<< '        existing_img_src = img_src_pattern_matches[1];' #save the parenthetical part of regexp for the first match
    #IE and Firefox behave differently, sometimes seem to return protocol + hostname in the src property
    #so, to make sure we don't have protocol and host name in front, do another step
    page_object<< '        img_src_pattern_matches = existing_img_src.match(/[^\/]+\/\/[^\/]+(\/.*$)/);'
    page_object<< '        if (img_src_pattern_matches) {' #if we found a match plus the saved parenthetized expression
    page_object<< '            existing_img_src = img_src_pattern_matches[1];' #save the parenthetical part of regexp (which should lop off the protocol + hostname)
    page_object<< '            }'
    page_object<< '        }'
    page_object<< '    if (existing_img_src != new_img_src) {' #if img src doesn't match (either because they're different or one was not found)
    page_object<< '        do_the_update = true;' #update
    page_object<< '        }'
    page_object<< '    if (do_the_update) {'
    page_object<< '        Element.update(table_element, replacementHTML);'
    page_object<< '        }'
    page_object<< '    }'
  end

  #========================================================================================================

  def bps_to_human_rate(bps)
    return "%d bps" % bps                         if bps < 1_000
    return "%.1f Kbps" % (bps/1_000.0)            if bps < 1_000_000
    return "%.1f Mbps" % (bps/1_000_000.0)        if bps < 1_000_000_000
    return "%.1f Gbps" % (bps/1_000_000_000.0)    if bps < 1_000_000_000_000
    return "%.1f Tbps" % (bps/1_000_000_000_000.0)
  rescue
    ""
  end

  def seconds_to_time(seconds)
    [seconds / 3600, seconds / 60 % 60, seconds % 60].map{|s| s.to_s.rjust(2, "0")}.join(":")
  rescue
    ""
  end

  def generate_form_content(specs)
    generate_form_content_helper(
                                 specs[:content],
                                 specs[:object_name],
                                 instance_variable_get("@#{specs[:object_name]}"))
  end

  # For merging html class options
  def add_option_or_default(current, default_option)
    current.present? ? "#{current} #{default_option}" : default_option
  end

  def generate_form_content_helper(content, object_name, record, level=0)
    _html = ''
    for _spec in content
      next if _spec.nil?
      if _spec[:legend]
        # generate fieldset
        _fs_html = content_tag('legend', _spec[:legend])
        _fs_html << content_tag('div', _spec[:description],
                                :class => 'description') if _spec[:description]
        _fs_html << generate_form_content_helper(_spec[:content], object_name,
                                                 record, level+1)
        _html << content_tag('fieldset', _fs_html, :class => "level#{level}", :id=>_spec[:id])
      else
        _spec[:row_options] = {} if _spec[:row_options].nil?
        _spec[:row_options][:class] = add_option_or_default(_spec[:row_options][:class], 'form_row')
        if _spec[:no_break]
          _html.slice!(-6,6)
          _html << generate_form_field(_spec, object_name, record) + '</div>'
        else
          _html << content_tag('div',
                               generate_form_field(_spec, object_name, record),
                               _spec[:row_options])
        end
      end
    end
    return _html
  end

  def generate_form_field(specs, object_name, record)
    _attribute = specs[:attribute]
    _value = specs[:value]
    _html = ''
    _options = {}
    _options.merge!(specs[:field_options]) if specs[:field_options]
    _width = specs[:width]

    if specs[:read_only]
      _value = record.send(_attribute) unless _value

      if specs[:type] == :datetime
        _value = Utils.time_to_human(_value)
      end

      _html << content_tag('p', _value)
    else
      case specs[:type]
      when :hidden
        _html << hidden_field(object_name, _attribute)
      when :text
        if _width
          _html << text_field(object_name, _attribute, :size=> _width, :maxlength => 255)
        else
          _html << text_field(object_name, _attribute, :maxlength => 255)
        end
      when :text_area
        if _width
          _html << text_area(object_name, _attribute, :rows => 3, :cols => _width, :maxlength => 1000)
        else
          _html << text_area(object_name, _attribute, :rows => 3, :maxlength => 1000)
        end
      when :boolean
        _checked_value = specs[:checked_value] ||= true
        _unchecked_value = specs[:unchecked_value] ||= false
        # Allow for class option
        _options[:class] = add_option_or_default(_options[:class], 'check_box')
        if _value.nil?
          _html << check_box(object_name, _attribute, _options, _checked_value,
                             _unchecked_value)
        else
          _name = "#{object_name}_#{_attribute}"
          _html << hidden_field_tag(_name, _unchecked_value, :id => nil) +
              check_box_tag(_name, _checked_value, _value == _checked_value, _options)
        end
      when :radio
        _html << radio_button(object_name, _attribute, specs[:button_value],
                              _options)
      when :password
        if _width
          _html << password_field(object_name, _attribute, :size => _width)
        else
          _html << password_field(object_name, _attribute)
        end
      else
        _html << content_tag('p', 'ERROR: type not recognized')
      end
    end

    if specs[:label]
      _for = ''
      _for = "#{object_name}_#{_attribute}" if _attribute
      if specs[:type] == :radio
        _html = content_tag(
                            'label',
                            "#{_html}#{specs[:label]}:",
                            :for => _for,
                            :class => 'radio')
      else
        _html = content_tag('label', "#{specs[:label]}:", :for => _for) + _html
      end
    end

    if specs[:suffix]
      _html << specs[:suffix]
    end

    if specs[:description]
      _html << content_tag('small', specs[:description])
    end

    return _html
  end

  def metadata_and_note(package, delivery)
    delivery = delivery.delivery if !delivery.is_a?(Delivery)
    data = ""
    if !delivery.user_metadata_fields.empty?
      data = "#{output_metadata(delivery || package)}"
    end
    note = h(delivery ? delivery.note : package.e_package_note).gsub(/\r\n/, '<br/>').gsub(/\n/, '<br/>')
    data << "<dt>" + t('helpers.application.note') + ":</dt><dd>#{note}</dd>" if note
    data
  end

  def output_metadata(delivery)
    mdata = ""
    delivery.user_metadata_fields.each do |key|
      mdata += "<dt>#{h(key)}:</dt><dd>#{h(delivery.metadata_hash[key])}&nbsp;</dd>"
    end
    mdata
  end

  def get_focus_field_from_form_specs(specs)
    _temp = specs
    while _temp[:content] && (_temp = _temp[:content][0])
      return "#{specs[:object_name]}_#{_temp[:attribute]}" if (_temp[:attribute])
    end
    return ''
  end

  def show_admin_layout?
    show_admin = %w[
      e_package
      post_processes
      ms_admin
      m_configuration
      m_template
      background_process
      metadata_profiles
      fasp_nodes
      source_nodes
      source_shares
      login_announcements
      package_instructions
      web_server
      display_settings
      package_storage
      transfer_options
      save_restore_config
      security
      content_security
      license
      authorization_domains
      saml_configurations
      entitlement
      global_distribution_lists
    ].include?(controller.controller_name)

    #only show the admin tabs if we're configuring AD
    show_admin |= (controller.controller_name == "ad_admin" && ["edit_config", "update_config"].include?(controller.action_name))
    show_admin
  end

  #-------------------------------------
  # package-specific wrapper functions
  #-------------------------------------

  def gr_pkg_date_sent(d)
    Utils.time_to_human(d.created_at)
  end

  def gr_sent_pkg_release_date(d)
    d.release_date.present? ? Utils.time_to_human(d.release_date) : gr_pkg_date_sent(d)
  end

  def gr_pending_pkg_release_date(d)
    d.release_date.present? ? Utils.time_to_human(d.release_date) : t('helpers.application.set_now')
  end

  def gr_pkg_released?(d)
    if d.release_policy == Delivery::RELEASE_POLICY_NOW
      true
    elsif d.release_date.present?
      d.release_date < Time.now
    else
      false
    end
  end

  def gr_pkg_upload_completion_time(p)
    p.e_package_upload_completion_time ? Utils.admin_time_to_human(p.e_package_upload_completion_time) : ""
  end

  def delete_confirmation_message(delivery)
    package = delivery.e_package
    message_prefix = ""
    if delivery.forward?
      message_prefix = t('helpers.application.delete_confirm_forward')
    elsif delivery.has_forwards?
      message_prefix = t('helpers.application.delete_confirm_has_forwards')
    end
    message_prefix + t('helpers.application.confirm', {package_name: package.e_package_name, package_sender: package.e_package_sender_name})
  end

  def gr_delivery_delete_files_link(delivery)
    package_record = delivery.e_package
    #leave link active no matter what the status of the files, in case the database record is incorrect due to previous delete failing (e.g. file was busy)
    if package_record.e_package_status != EPackage::STATUS_VALID ||
        package_record.e_package_files_deleted_by_admin_time ||
        !package_record.inner_directory_exists?
      return ''
    end

    link_to(
      t('helpers.application.delete'),
      {:action => 'delete_files', :id => package_record.id}.merge(package_list_params(params)),
      :id => "delete_link_#{package_record.id}",
      :confirm => delete_confirmation_message(delivery),
      :method => :post
    )
  end

  def validation_failed?(downloadable_object)
    downloadable_object.e_package.e_package_upload_status == ESession::STATUS_VALIDATION_FAILED
  end

  def gr_pkg_dl_icon_small downloadable_object
    generate_image_download_link downloadable_object
  end

  def gr_pkg_file_status(downloadable_object)
    if downloadable_object.is_a?(EPackage) && downloadable_object.e_package_status == EPackage::PENDING_DELETION
      return 'deleting'
    elsif downloadable_object.is_a?(EPackage) && downloadable_object.e_package_files_deleted_by_admin_time
      return 'deleted'
    elsif !downloadable_object.inner_directory_exists?
      return 'no'
    elsif !downloadable_object.is_a?(EPackage) || downloadable_object.e_package_contents_exist?
      if downloadable_object.completed?
        return 'yes'
      else
        return 'partial'
      end
    else
      return 'no'
    end
  end

  def gr_pkg_remove_link(package)
    return "" if package.busy_uploading?
    link_to(t('helpers.application.archive'),
            { :action => 'remove_from_view', :id => package.id },
            { :method => :post,              :id => "remove_link_#{package.id}" })
  end

  def gr_del_remove_link(delivery)
    return "" if delivery.busy_uploading?
    link_to(t('helpers.application.archive'), { :action => 'remove_from_view', :id => delivery.id }.merge(package_list_params(params)),
                       { :method => :post, :id => "remove_link_#{delivery.id}"})
  end

  def gr_pkg_remove_link_wg(package)
    return "" if package.busy_uploading?
    #otherwise, show link only if I'm an admin of this workgroup, or if I'm system admin
    logged_in_user = controller.current_user
    if logged_in_user.admin? || logged_in_user.workgroup_admin?(session[:selected_workgroup])
      return link_to(t('helpers.application.archive'), { :action => 'remove_from_view', :id => package.id },
                                { :method => :post,              :id => "remove_link_#{package.id}" })
    else
      return ''
    end
  end

  def gr_pkg_row_class(package)
    #controls bolding or dimming of row
    if package.e_package_files_deleted_by_admin_time ||
        !package.inner_directory_exists? ||
        !package.e_package_contents_exist ||
        package.e_package_status != EPackage::STATUS_VALID
      return 'invalid'
    else
      return ''
    end
  end

  def gr_pkg_total_size_bytes(p)
    if (p.e_package_total_size_bytes == 0) && (p.e_package_total_files != 0)
      number_to_human_size(p.calculate_size("/"))
    else
      number_to_human_size(p.e_package_total_size_bytes)
    end
  end

  def gr_delivery_total_size_bytes(delivery)
    return "--" if delivery.forward?
    return gr_pkg_total_size_bytes(delivery.e_package)
  end

  def gr_pkg_total_files(p)
    p.e_package_total_files
  end

  def gr_failed_pkg_total_files(p)
    p.calculate_file_count("/")
  end

  def gr_failed_pkg_total_size_bytes(p)
    number_to_human_size(p.calculate_size("/"))
  end

  def gr_pkg_dl_stats(p)
    "#{p.get_total_completed_downloads}/#{p.get_total_completed_partial_downloads}"
  end

  def gr_pkg_retry_stop_icon_small(downloadable_object, pkg_passcode = nil)
    package = downloadable_object.e_package

    #check to make sure it wasn't deleted, removed by sender, or an http_gateway transfer
    return "" if package.e_package_files_deleted_by_admin_time ||
                 package.e_package_removed_by_sender ||
                  package.http_gateway_transfer

    cell_id = "cell_action_icon_#{downloadable_object.id}"
    url_params =  pkg_passcode.present? ? {package: pkg_passcode} : {id: downloadable_object.id}

    case downloadable_object.upload_status
    when ESession::STATUS_SUBMITTED, ESession::STATUS_TO_BE_VALIDATED, ESession::STATUS_VALIDATING, ESession::STATUS_VALIDATION_TIMED_OUT then ""
    when ESession::STATUS_TIMED_OUT, ESession::STATUS_ERROR, ESession::STATUS_STOPPED
      (package.source_share && !package.source_share.remote?) ? retry_upload_icon(cell_id, url_params) : ""
    when ESession::STATUS_TRANSFERRING
      (pkg_passcode.present? ? "" : stop_upload_icon(cell_id, url_params))
    else ""
    end
  end

  def stop_upload_icon(cell_id, url_params)
    link_to_remote(
      image_tag("stop.gif", alt: t('helpers.application.stop_upload'), title: t('helpers.application.stop_upload')),
      :url    => { :action => "ajax_stop_upload" }.merge(url_params),
      :before => %Q{Element.update('#{cell_id}','')}
    )
  end

  def retry_upload_icon(cell_id, url_params)
    link_to_remote(
      image_tag("retry.gif", alt: t('helpers.application.retry_upload'), title: t('helpers.application.stop_upload')),
      :url    => { :action => "ajax_retry_package_upload" }.merge(url_params),
      :before => "Element.update('#{cell_id}','')"
    )
  end

  def package_list_params(params)
    params[:page] = params[:page].to_i == 0 ? nil : params[:page].to_i

    {
        :a_order => sanitize(params[:a_order]),
        :a_sort => sanitize(params[:a_sort]),
        :page => params[:page],
        :search => sanitize(params[:search])
    }.delete_if{|k,v| v.blank?}
  end

  def gr_del_title_link(delivery_record)
    #max_length = 40

    # get package name, limit max_length
    name = delivery_record.title
    #truncate if too long
    #if (name.length > max_length)
    #  name = name[0..(max_length-1)] + '...'
    #end
    if delivery_record.forward?
      name = EPackage::FORWARDED_PACKAGE_PREFIX + name
    end
    link_to(h(name), { :action => 'show', :id => delivery_record.id, :params => package_list_params(params) }, :alt => t('helpers.application.package_details'), :id => "del_link_#{delivery_record.id}" )
  end

  def gr_sent_title_link(send_record)
    name = send_record.title
    if send_record.forward?
      name = EPackage::FORWARDED_PACKAGE_PREFIX + name
    end
    link_to(h(name), { :action => 'show', :id => send_record.id, :params => package_list_params(params) }, :alt => t('helpers.application.package_details'), :id => "pkg_link_#{send_record.id}" )
  end

  def gr_del_title_link_from_history(delivery_record)
    name = delivery_record.title
    if delivery_record.forward?
      name = EPackage::FORWARDED_PACKAGE_PREFIX + name
    end
    # generate html for name
    link_to(h(name), {:action => 'show', :id => delivery_record.id, :view => 'history', :params => package_list_params(params)}, :alt => t('helpers.application.package_details'), :id => "del_link_#{delivery_record.id}")
  end

  def gr_sent_title_link_from_history(sent_record)
    name = sent_record.title
    if sent_record.forward?
      name = EPackage::FORWARDED_PACKAGE_PREFIX + name
    end
    # generate html for name
    link_to(h(name), {:action => 'show', :id => sent_record.id, :view => 'history', :params => package_list_params(params)}, :alt => t('helpers.application.package_details'), :id => "pkg_link_#{sent_record.id}")
  end

  def gr_pkg_title_link(package_record)
    name = package_record.e_package_name
    link_to(h(name), {:action => 'show', :id => package_record.id, :params => package_list_params(params)}, :id => "pkg_link_#{package_record.id}")
  end

  def gr_delivery_title_link(delivery)
    name = delivery.title
    link_to(h(name), {:action => 'show', :id => delivery.id, :params => package_list_params(params)}, :id => "pkg_link_#{delivery.e_package.id}")
  rescue
    ''
  end

  def gr_pkg_title_link_from_history(package_record)
    name = package_record.e_package_name
    link_to(h(name), {:action => 'show', :id => package_record.id, :view => 'history', :params => package_list_params(params)})
  end

  def gr_delivery_upload_progress(delivery, page = 1)
    return gr_delivery_relay_progress(delivery, page) if delivery.has_relays?
    return upload_status(delivery.e_package)
  end

  def gr_delivery_relay_progress(delivery, page)
    link_to(delivery.pretty_relay_status, :action => 'relay_details', :id => delivery.id, :params => package_list_params(params))
  end

  def received_upload_status(user_delivery)
    user_delivery.delivery.upload_status_message
  end

  # Return display text to show upload status of a package, delivery, or recipient_delivery.
  def upload_status(uploadable_object, force_status = false)
    text = uploadable_object.upload_status_message
    if uploadable_object.upload_status == ESession::STATUS_TRANSFERRING
      text += transfer_progress_message(uploadable_object.upload_statistics)

      package = nil
      if uploadable_object.is_a?(Delivery)
        package = uploadable_object.e_package
      elsif uploadable_object.is_a?(RecipientDelivery)
        package = uploadable_object.delivery
      elsif uploadable_object.is_a?(EPackage)
        package = uploadable_object
      end

    end
    text
  end

  def transfer_progress_message(stats)
    if stats[:progress_percentage] && stats[:seconds_remaining] && stats[:bytes_total]
      "<br />#{number_to_human_size(stats[:bytes_written])}/#{number_to_human_size(stats[:bytes_total])}" +
          "<br />(#{number_to_percentage(stats[:progress_percentage], :precision => 1)})" +
          "<br />#{seconds_to_time(stats[:seconds_remaining])} remaining"
    else
      ''
    end
  end

  #===========================================================================================

  # SHARED PACKAGE FORM GENERATOR HELPERS


  # Wrap download image in anchor tag so it will be detected as an action by screenreaders
  def generate_image_download_link(downloadable_object, large = false, path_in_pkg = nil,  folder = false, fasp_file = nil, relative_path = nil)
    return "<a>#{generate_image_download(downloadable_object, large, path_in_pkg, folder, fasp_file, relative_path)}</a>"
  end

  # downloadable_object - delivery or recipient_delivery
  # large (optional) - true to use large icons
  # path_in_pkg (optional) - specify a path (within the package) to download
  #   specific file or folder; defaults to path of entire package
  # folder (optional) - true to use folder icon vs file icon (if path is set)
  # fasp_file (optional) - the fasp_file of the path to download
  def generate_image_download(downloadable_object, large = false, path_in_pkg = nil, folder = false, fasp_file = nil, relative_path = nil)
    return '' unless current_user.downloads_allowed?
    downloadable_object = downloadable_object.delivery if downloadable_object.is_a?(RecipientDelivery) && downloadable_object.relay_failed?
    package = downloadable_object.e_package
    alt_text = path_in_pkg.present? ? "File: #{path_in_pkg}" : "Package: #{downloadable_object.e_package.e_package_name}"

    # Normal case handling - return large gray error icons, spinners, blank
    sess_stat = downloadable_object.upload_status
    if sess_stat == ESession::STATUS_SUBMITTED
      return image_tag 'folder_loading.gif', alt: "Submitted #{alt_text}", class: 'folder_transferring_spinner' if large
      return ''
    elsif [ESession::STATUS_TRANSFERRING, ESession::STATUS_TO_BE_VALIDATED, ESession::STATUS_VALIDATING].include?(sess_stat)
      return image_tag 'folder_loading.gif', alt: "Transferring #{alt_text}", class: 'folder_transferring_spinner'
    elsif [ESession::STATUS_ERROR, ESession::STATUS_TIMED_OUT, ESession::STATUS_STOPPED, ESession::STATUS_VALIDATION_TIMED_OUT].include?(sess_stat)
      return image_tag '48parcel_error_gray.gif', alt: "Error #{alt_text}" if large
    elsif (sess_stat == ESession::STATUS_VALIDATION_FAILED) || (package.validation_status == EPackage::VALIDATION_FAILED)
      return image_tag '48parcel_error_gray.gif', alt: "Error #{alt_text}" if large
    elsif sess_stat != ESession::STATUS_COMPLETE # Return blank string for all invalid session statuses
      return ''
    end

    if permit_otfv_downloads(downloadable_object, package, fasp_file) && permit_downloads(downloadable_object, sess_stat, path_in_pkg)
      image_filename = large ? '48parcel_download.gif' : 'download.gif'
      return image_tag image_filename,
        class: 'download-file',
        alt: "Download #{alt_text}",
        'data-relative_path' => relative_path || '/',
        'data-path' => path_in_pkg || '/',
        'data-download_url' => polymorphic_url([scope, downloadable_object], action: :download, passcode: params[:passcode])
    else
      return ''
    end
  end

  # Handle OTFV - block failed downloads
  def permit_otfv_downloads(downloadable_object, package, fasp_file = nil)
    allow_download = downloadable_object.upload_status == ESession::STATUS_COMPLETE # Allow downloads if package transferred successfully
    allow_download = true if ((fasp_file != nil) and (fasp_file.status == FaspFile::STATUS_COMPLETED)) # Also allow downloads if the fasp_file's status is complete
    allow_download = false if (package.validation_status == EPackage::VALIDATION_FAILED) and (fasp_file == nil) # But block download if fasp_file expired and package failed
    return allow_download
  end

  def permit_downloads(downloadable_object, sess_stat, path_in_pkg = nil)
    # Handle download icons/link - arrived here if ((ERROR, TIMED_OUT, STOPPED, or VALIDATION_FAILED) but not large), or COMPLETED
    can_download = false
    if sess_stat == ESession::STATUS_COMPLETE
      can_download = false if !downloadable_object.inner_directory_exists? || downloadable_object.e_package.e_package_status == EPackage::PENDING_DELETION
      can_download = true
    else
      # we're relying on the helper that calls this not to show download icons for files that are only partially transferred
      can_download = true if path_in_pkg
    end
    can_download = false if downloadable_object.delivery.prevent_http_download && connect_disabled?
    return can_download
  end

  def generate_download_link(downloadable_object, path_in_pkg = nil, folder = false, fasp_file = nil, relative_path = nil)
    return '' unless current_user.downloads_allowed?
    downloadable_object = downloadable_object.delivery if downloadable_object.is_a?(RecipientDelivery) && downloadable_object.relay_failed?
    package = downloadable_object.e_package
    alt_text = path_in_pkg.present? ? "File: #{path_in_pkg}" : "Package: #{downloadable_object.e_package.e_package_name}"

    sess_stat = downloadable_object.upload_status
    # Return blank string for all invalid session statuses
    if sess_stat != ESession::STATUS_COMPLETE || (package.validation_status == EPackage::VALIDATION_FAILED)
      return ''
    end

    if permit_otfv_downloads(downloadable_object, package, fasp_file) && permit_downloads(downloadable_object, sess_stat, path_in_pkg)
      return "<a> #{link_to download_link_text(downloadable_object), '#',
                            class: 'download-file',
                            alt: "Download #{alt_text}",
                            'data-relative_path' => relative_path || '/',
                            'data-path' => path_in_pkg || '/',
                            'data-download_url' => polymorphic_url([scope, downloadable_object], action: :download, passcode: params[:passcode])}</a>"
    else
      return ''
    end
  end

  def generate_name_and_note(package)
    max_length = 40

    # get package name, limit max_length
    name = package.e_package_name
    name = name[0...max_length] + '...' if (name.length > max_length)

    link_to(name, {:action => 'show', :id => package.id})
  end

  def get_toolbar_path(relative_path)
    toolbar_html = [t('app.helpers.application_helper.path')]
    breadcrumbs = package_breadcrumbs(relative_path)
    breadcrumbs.each do |breadcrumb|
      toolbar_html << ' / ' <<
          breadcrumb_html(breadcrumb, breadcrumb == breadcrumbs.last)
    end
    toolbar_html.join
  end

  def package_breadcrumbs(path)
    segments = path.sub(%r{^/}, '').split('/').unshift('')
    cumulative_path = ''
    segments.map do |segment|
      cumulative_path += (segment + '/')
    end
  end

  def breadcrumb_html(path, hide_link)
    if hide_link
      "<span class='path_part'>#{human_basename(path)}</span>"
    else
      link_to_remote(human_basename(path),
                     { url: {
                         action:   "ajax_update_file_browser",
                         id:       params[:id],
                         new_path: path,
                         passcode: params[:passcode]
                       },
                       :html => { :class => "path_part" },
                       :before => "aspera_cancel_update()",
                       :complete => "aspera_start_update('#{escape_javascript(path)}')"
                     })
    end
  end

  def human_basename(path)
    if path == '/'
      @package.inner_directory_name
    else
      File.basename(path)
    end
  end

  def pagination_settings
    { :inner_window => 2, :outer_window => 2 }
  end

  def body_data
    {
      'data-preferences' =>
        ActiveSupport::JSON.encode({
          suppress_connect_install_dialog: current_user.present? ? current_user.suppress_connect_install_dialog? : true
        }),
      'data-global-settings' =>
        ActiveSupport::JSON.encode({
          minimum_connect_version:     CConfiguration.get_minimum_connect_version,
          prefer_http_connect:         CConfiguration.first.prefer_http_connect,
          disable_http_transfer:       !!FASPEX_CONFIG['DisableHttpTransfer']
        })
    }
  end

  def format_percentage(percentage)
    number_with_precision(percentage, :precision => 0) || ""
  end

  def parent_directory(path)
    Pathname.new(path).parent.to_s
  end

  def flash_divs
    String.new.tap do |s|
      flash.each do |name, msg|
        s << content_tag(:div, :class => "alert alert-block alert-#{name}") do
          content_tag :p, h(msg)
        end
      end
    end.html_safe
  end

  # @remark - Hashes are ordered in ruby 1.9
  def browser_sort_options
    @browser_sort_options ||= ActiveSupport::OrderedHash.new.tap do |map|
      map["Name"] = "path"
      map["Type"] = "type"
      map["Largest first"] = "size_d"
      map["Smallest first"] = "size_a"
      map["Newest first"] = "mtime_d"
      map["Oldest first"] = "mtime_a"
    end
  end

  # @remark - Find a logical way to combine this method with the last method.
  def browser_headers
    @browser_headers ||= ActiveSupport::OrderedHash.new.tap do |map|
      map["Name"] = %w[path]
      #map["Type"] = "type"
      map["Size"] = %w[size_d size_a]
      map["Last modified <span class='dim'>in UTC</span>"] = %w[mtime_d mtime_a]
    end
  end

  def browse_path(browseable_object, _params = {})
    if controller.class.name.split("::").first == "Admin"
      polymorphic_path([:browse, :admin, browseable_object], _params)
    else
      polymorphic_path([:browse, browseable_object], _params)
    end
  end

  def browseable_object_path(browseable_object, path)
    if browseable_object.kind_of?(Share)
      browseable_object.share_path(path)
    else
      path
    end
  end

  def breadcrumbs(browseable_object, current_path)
    divider = content_tag(:span, '/', :class => 'divider')

    Pathname.new(current_path).to_enum(:descend).map do |path|

      text = path == Pathname('/') ? browseable_object.name : path.basename

      text = h(truncate(text.to_s, :length => 65)).gsub(/\s/, '&nbsp;')

      if current_path.to_s == path.to_s
        content_tag(:li, text)
      else
        url = browse_path(browseable_object, :path => path)
        content_tag :li, link_to_remote(text, :url => url, :method => :get)
      end
    end.join(divider)
  end

  def password_strength_text
    ERB::Util.h User::Passwordable.password_requirements.capitalize
  end

  def node_symlink_support_text(supported)
    supported ? t('helpers.application.supp_symbolic_links') : t('helpers.application.not_supp_symbolic_links')
  end

  def cloud_referencing_supported?(share)
    (share.uses_azure_storage? || share.uses_s3_storage?)
  end

  def cloud_referencing_support_text(share)
    if cloud_referencing_supported?(share)
      text = t('helpers.application.cloud_storage_referencing')
    else
      text = t('helpers.application.no_cloud_storage_referencing')
    end

    text
  end

  # link_to 'back', :back uses the referer header, but fails to protect against XSS attacks.
  # This is used to obtain a safe back_url based off the referer header that falls back to a specified URL.
  def back_url(fallback_url = root_url)
    referer_url = request.env['HTTP_REFERER']

    begin
      URI.parse(referer_url) # raises URI::InvalidURIError for invalid URIs used in XSS attacks
    rescue URI::InvalidURIError
      return fallback_url
    end

    referer_url
  end

  def h1 text, options = {}
    if options[:back].present?
      content_tag :h1 do
        text + ' ' + (content_tag :small, link_to(t('helpers.application.back'), options[:back], id: 'back_link'))
      end
    else
      content_tag :h1, text
    end
  end

  def slave?
    @is_slave ||= HomeServer.this_server.slave?
  end

  # Adjusts the start year for date select
  def start_year(selected_time = nil)
    if selected_time.present?
      [selected_time.year, Time.now.year].min
    else
      Time.now.year
    end
  end

  # Adjusts the end year for date select
  def end_year
    Time.now.year + 5
  end

  def user_settings_view?
    controller.controller_path =~ /settings/ || controller.controller_name == 'contacts'
  end

  def server_settings_view?
    controller.controller_path =~ %r{admin/settings}
  end

  def server_notifications_view?
    controller.controller_path =~ %r{admin/notifications} || %w[m_configuration m_template].include?(controller.controller_name)
  end

  def authentication_view?
    controller.controller_path =~ %r{admin/authorization_domains|admin/saml}
  end

  def error_view?
    controller.controller_name == 'e_error'
  end

  def has_side_view?
    user_settings_view? || server_settings_view? || server_notifications_view? || authentication_view? ||
        %w[ms_admin].include?(controller.controller_name)
  end

  def public_url_copy_field user_or_dropbox
    url_id = 'public_url'
    url = vanity_url user_or_dropbox

    text_field = text_field_tag :public_url, url, id: url_id, readonly: 'readonly', style: 'color: gray; width: 364px;', class: 'select-all-on-click'

    t('helpers.application.public_url') + " #{text_field} #{image_tag 'file3.gif', id: 'copy-button', 'data-target' => url_id, title: t('helpers.application.copy')}"
  end

  def http_transfer_message direction
    connect_install_message(direction) + connect_upgrade_message(direction)
  end

  def connect_transfer_message direction
    connect_upgrade_message direction
  end

  def connect_upgrade_message direction
    content_tag(:div, class: "connect-upgrade hidden_element alert alert-warning", id: "upgrade-plugin-warning", style: "width: 95%") do
      t('helpers.application.connect_upgrade_message', {direction: direction, aspera: (link_to 'Aspera Connect', '#', onclick: 'window.location = window.connect_download_link; return false;')})
    end
  end

  def connect_install_message direction
    content_tag(:div, class: "connect-install hidden_element alert alert-warning", id: "plugin-required-warning", style: "width: 95%") do
      t('helpers.application.connect_install_message', {direction: direction, aspera: (link_to 'Aspera Connect', '#', onclick: 'window.location = window.connect_download_link; return false;')})
    end
  end

  # Warning message in send/new
  def no_transfer_message direction
    content_tag(:div, class: "connect-install hidden_element alert alert-warning", id: "plugin-required-warning", style: "width: 95%") do
      download_link = link_to 'download', '#', :onclick => 'window.location = window.connect_download_link; return false;'
      if direction == 'upload'
        if current_user && current_user.disable_connect
          t('helpers.application.no_transfer_dl_enable', {download: download_link, enable: (link_to 'enable', edit_settings_preferences_path(anchor: :misc))})
        else
          t('helpers.application.no_transfer_dl', {download: download_link})
        end
      else
        if current_user && current_user.disable_connect
          t('helpers.application.enable_hst_dl_enable', {download: download_link, enable: (link_to 'enable', edit_settings_preferences_path(anchor: :misc))})
        else
          t('helpers.application.enable_hst_dl', {download: download_link})
        end
      end
    end
  end

  # Warning message in /sent
  def local_http_transfer_message
    t('helpers.application.local_transfer_message', {enable: (link_to 'User Preferences', edit_settings_preferences_path)})
  end

  def connect_version_message
      "(min. version #{CConfiguration.get_minimum_connect_version})"
  end

  def space_free_message
    bytes_free = number_to_human_size(StorageShare.first.bytes_free)
    if bytes_free
      "#{bytes_free} remaining for packages"
    else
      "" # suppress bytes free message if unavailable
    end
  end

  # Warning - More than one per page will cause unintended behavior
  def pretty_browse(browse_opts = {}, button_opts = {}, file_list_opts = {})
    button_opts[:class] = "pretty-browse-button btn #{button_opts[:class]}"
    button_opts[:type] = 'button'
    button_opts[:label] = t('app.helpers.application_helper.browse') unless button_opts[:label].present?
    browse_opts[:class] = "pretty-browser hidden #{browse_opts[:class]}"
    file_list_opts[:class] = "pretty-browse-files #{file_list_opts[:class]}"
    render :partial => 'layouts/pretty_browse', :locals => {:browse_opts => browse_opts, :button_opts => button_opts, :file_list_opts => file_list_opts}
  end

  def compare_version_strings(s1, s2)
    s1 = s1.split('.').map(&:to_i)
    s2 = s2.split('.').map(&:to_i)

    if s1[0] > s2[0]
      return 1
    elsif s1[0] == s2[0]
      if s1[1] > s2[1]
        return 1
      elsif s1[1] == s2[1]
        if s1[2] > s2[2]
          return 1
        elsif s1[2] == s2[2]
          return 0
        end
      end
    end

    return -1
  end

  def current_custom_logo_path
    return 'custom/custom_logo.png' if File.exists?(Rails.root.join('public', 'images', 'custom', 'custom_logo.png'))
    return 'custom/custom_logo.jpg' if File.exists?(Rails.root.join('public', 'images', 'custom', 'custom_logo.jpg'))
    return 'custom/custom_logo.jpeg' if File.exists?(Rails.root.join('public', 'images', 'custom', 'custom_logo.jpg'))

    'custom/custom_logo.gif' if File.exists?(Rails.root.join('public', 'images', 'custom', 'custom_logo.gif'))
  end

  def current_tab
    path = request.path

    if /\/send\/new/.match(path) || /\/dropboxes\/\d+\/deliveries\/new/.match(path)
      'new_package'
    elsif /\/sent/.match(path)
      'sent'
    elsif /\/pending/.match(path)
      'pending'
    elsif /\/received/.match(path) || /\/invitations/.match(path)
      'received'
    elsif /\/workgroups/.match(path) || /\/dropboxes/.match(path)
      'workgroups'
    elsif /\/account/.match(path)
      'accounts'
    else
      'server'
    end
  end

  def tab_selected?(tab)
    @current_tab ||= current_tab
    current_tab == tab ? 'selected' : ''
  end
end
