class ExternalUser < User
  attr_accessible :name, :password, :first_name, :last_name, :email,
      :needs_sync_to_master, :creator_name, :disable_connect

  def external_user_ear
    self.received_invitations.first.ear
  end

  def name
    if read_attribute(:email).present?
      read_attribute(:email)
    else
      read_attribute(:name)
    end
  end

  def display_name
    name
  end

  def email
    if read_attribute(:email).present?
      read_attribute(:email)
    else
      name
    end
  end

  def first_name
    name
  end

  def last_name
    "(external email)"
  end

  def upgrade
    self.type = LocalUser.name
  end

  def kbps_initial_target_upload_rate
    conf.c_kbps_initial_target_upload_rate
  end

  def kbps_initial_target_download_rate
    conf.c_kbps_initial_target_download_rate
  end

  def kbps_max_target_upload_rate
    conf.c_kbps_max_target_upload_rate
  end

  def kbps_max_target_download_rate
    conf.c_kbps_max_target_download_rate
  end

  def lock_minrate_policy
    conf.c_lock_minrate_policy
  end

  def self.upgrade_to_remote_user(user, attributes)
    User.transaction do
      # change user to a LocalUser
      user.type = DirectoryServiceUser.name
      user.save
      user = DirectoryServiceUser.find(user.id)

      unless user.update_attributes(attributes)
        raise ActiveRecord::Rollback
      end

      # Convert recipient name in user deliveries in case username has changed or external flag is present.
      UserDelivery.find_all_by_recipient_id(user.id).map{ |d| d.update_attribute(:recipient_name, user.name)}
    end
    user
  end

  # upgrade the specified external user and return the successfully created
  # upgraded user or failed user with errors
  def self.upgrade(user, attributes)
    User.transaction do
      # change user to a LocalUser
      user.type = LocalUser.name
      user.force_password_reset = EConfiguration.first.force_password_reset_for_new_users
      user.save
      user = LocalUser.find(user.id)

      unless user.update_attributes(attributes)
        raise ActiveRecord::Rollback
      end

      # Convert recipient name in user deliveries in case username has changed or external flag is present.
      UserDelivery.find_all_by_recipient_id(user.id).map{ |d| d.update_attribute(:recipient_name, user.name)}
    end
    user
  end

private

  def conf
    @conf ||= CConfiguration.first
  end

end
