class Notifier < ActionMailer::Base
  @@helper = Object.new.extend(ActionView::Helpers::NumberHelper)

  #=============================================================
  #
  # CLASS METHODS
  #
  #=============================================================

  def self.configure_mailer
    mail_conf = MConfiguration.first
    case mail_conf.m_smtp_auth_type
    when "open" # open relay
      configure_open_smtp(mail_conf)
    when "login" # SMTP login
      configure_login_smtp(mail_conf)
    else
      raise ArgumentError, "Unknown mail_smtp_auth_type '" + mail_conf.m_smtp_auth_type.to_s + "'"
    end
  end

  private

  def self.configure_open_smtp(mail_conf)
    ActionMailer::Base.delivery_method = :smtp
    ActionMailer::Base.smtp_settings = {
      :enable_starttls_auto => mail_conf.use_tls?,
      :address => mail_conf.m_smtp_server,
      :port => mail_conf.m_smtp_server_port,
      :domain => mail_conf.m_smtp_helo_domain,
      :authentication => :plain
    }
  end

  def self.configure_login_smtp(mail_conf)
    ActionMailer::Base.delivery_method = :smtp
    ActionMailer::Base.smtp_settings = {
      :enable_starttls_auto => mail_conf.use_tls?,
      :address => mail_conf.m_smtp_server,
      :port => mail_conf.m_smtp_server_port,
      :domain => mail_conf.m_smtp_helo_domain,
      :authentication => :login,
      :user_name => mail_conf.m_smtp_username,
      :password => mail_conf.unencrypted_smtp_password
    }
  end

  def self.mail_configuration
    MConfiguration.find(:first)
  end

  def self.faspex_name
    mail_configuration.email_from
  end

  def self.faspex_email
    MConfiguration.from_header_email
  end

  def self.faspex_from_header
    "#{faspex_name} <#{faspex_email}>"
  end

  def self.package_received_from_header(sender)
    case mail_configuration.package_received_notifications_from
      when MConfiguration::FASPEX then faspex_from_header
      when MConfiguration::SENDER then sender.full_email_address_descriptor
      when MConfiguration::SENDER_VIA_FASPEX then "#{sender.full_name_untruncated} via #{faspex_from_header}"
      when MConfiguration::SENDER_EMAIL then sender.email_address_descriptor
      else raise "Invalid value for \"From:\" header"
    end
  end

  def self.package_from(sender)
    case mail_configuration.package_received_notifications_from
      when MConfiguration::FASPEX then sender.full_name_untruncated
      when MConfiguration::SENDER then sender.full_email_address_descriptor
      when MConfiguration::SENDER_VIA_FASPEX then "#{sender.full_name_untruncated} via #{faspex_from_header}"
      when MConfiguration::SENDER_EMAIL then sender.email_address_descriptor
    end
  end

  public

  #=============================================================
  #
  # INSTANCE METHODS
  #
  #=============================================================

  def package_downloaded_notification(recipient, package, delivery, package_url, partial_package, notified_user)
    Notifier.configure_mailer()
    _downloaded_template = MTemplate.find_by_name('package_downloaded')
    _downloaded_template.generate_rhtml(false)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)
    # sub in Faspex variables
    if recipient == notified_user
      _sender = User.find(recipient.id)
    else
      _sender = User.find(delivery.sender_id)
    end
    _subject = _downloaded_template.subject
    _subject.gsub!('MOBILE_PACKAGE_URL', package_url)
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', package.e_package_uuid)
    _subject.gsub!('DOWNLOADER_NAME', recipient.full_name_untruncated)
    _subject.gsub!('DOWNLOADER_FIRST_NAME', recipient.first_name)
    _subject.gsub!('DOWNLOADER_LAST_NAME', recipient.last_name)
    _subject.gsub!('DOWNLOADER_EMAIL', recipient.email)
    _subject.gsub!('DOWNLOADER_LOGIN', recipient.name)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('PACKAGE_URL', package_url)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', package.e_package_total_files.to_s + " file" + (package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.note)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(recipient, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(recipient, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))
    self.subject(_subject)
    self.recipients(notified_user.full_email_address_descriptor)
    self.sent_on(Time.now)

    content_type "multipart/alternative"

    _body_vars = {
      :package_name => delivery.title,
      :package_uuid => package.e_package_uuid,
      :package_url => package_url,
      :sender_name => _sender.full_name_untruncated,
      :sender_first_name => _sender.first_name,
      :sender_last_name => _sender.last_name,
      :sender_email => _sender.email,
      :sender_login => _sender.name,
      :package_date => delivery.created_at,
      :package_size => package.e_package_total_size_bytes,
      :package_files => package.e_package_total_files,
      :package_partial => partial_package.blank? ? "" : "one or more files from",
      :package_file_list_first_10 => package.file_list_first_10,
      :package_note => delivery.metadata_and_note,
      :downloader_email => recipient.email,
      :downloader_name => recipient.full_name_untruncated,
      :downloader_first_name => recipient.first_name,
      :downloader_last_name => recipient.last_name,
      :downloader_login => recipient.name,
      :all_public_recipients => delivery.public_recipients.join(', '),
      :all_public_recipients_email => delivery.public_recipients_email.join(', '),
      :all_cc_recipients => all_cc_recipients(recipient, delivery).join(', '),
      :all_cc_recipients_email => all_cc_recipients_email(recipient, delivery).join(', '),
      :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("package_downloaded_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("package_downloaded_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, _downloaded_template.include_alternate_address_content)
  end

  def package_downloaded_cc_notification recipient, package, delivery, notification
    Notifier.configure_mailer()
    _downloaded_template = MTemplate.find_by_name('package_downloaded_cc')
    _downloaded_template.generate_rhtml(false)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)
    # sub in Faspex variables
    _sender = User.find(delivery.sender_id)


    _subject = _downloaded_template.subject
    _subject.gsub!('CC_NAME', notification.name)
    _subject.gsub!('CC_EMAIL', notification.email)
    _subject.gsub!('DOWNLOADER_NAME', recipient.full_name_untruncated)
    _subject.gsub!('DOWNLOADER_FIRST_NAME', recipient.first_name)
    _subject.gsub!('DOWNLOADER_LAST_NAME', recipient.last_name)
    _subject.gsub!('DOWNLOADER_EMAIL', recipient.email)
    _subject.gsub!('DOWNLOADER_LOGIN', recipient.name)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', package.e_package_uuid)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', package.e_package_total_files.to_s + " file" + (package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.note)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(recipient, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(recipient, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))

    self.subject(_subject)
    self.recipients(notification.email)
    self.sent_on(Time.now)

    content_type "multipart/alternative"

    _body_vars = {
      :cc_name => notification.name,
      :cc_email => notification.email,
      :downloader_email => recipient.email,
      :downloader_name => recipient.full_name_untruncated,
      :downloader_first_name => recipient.first_name,
      :downloader_last_name => recipient.last_name,
      :downloader_login => recipient.name,
      :sender_name => _sender.full_name_untruncated,
      :sender_first_name => _sender.first_name,
      :sender_last_name => _sender.last_name,
      :sender_email => _sender.email,
      :sender_login => _sender.name,
      :package_name => delivery.title,
      :package_uuid => package.e_package_uuid,
      :package_date => delivery.created_at,
      :package_size => package.e_package_total_size_bytes,
      :package_files => package.e_package_total_files,
      :package_file_list_first_10 => package.file_list_first_10,
      :package_note => delivery.metadata_and_note,
      :all_public_recipients => delivery.public_recipients.join(', '),
      :all_public_recipients_email => delivery.public_recipients_email.join(', '),
      :all_cc_recipients => all_cc_recipients(recipient, delivery).join(', '),
      :all_cc_recipients_email => all_cc_recipients_email(recipient, delivery).join(', '),
      :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("package_downloaded_cc_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("package_downloaded_cc_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, _downloaded_template.include_alternate_address_content)
  end


  def relay_failed_notification(relay)
    relay_failed_template = MTemplate.find_by_name('relay_failed')
    relay_failed_template.generate_rhtml(false)
    delivery = relay.delivery
    home_server = relay.home_server
    recipients = delivery.recipients_for_home_server(home_server)
    recipients_csv = recipients.collect(&:addressable_name).join(", ")
    if delivery.sender.external_email_user?
      package_url = HomeServer.this_server.url +
          external_submission_status_path(package: delivery.dbox_extusr_sender_pkg_passcode)
    else
      package_url = HomeServer.this_server.url + sent_path(delivery, local: delivery.sender.force_local_login?)
    end

    Notifier.configure_mailer()
    email_recipient = User.find_by_name(delivery.sender_name)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(relay_failed_template.subject.
                 gsub(/DEST_HOME_SERVER_NAME/, home_server.name).
                 gsub(/SENDER_NAME/, email_recipient.full_name_untruncated).
                 gsub(/SENDER_FIRST_NAME/, email_recipient.first_name).
                 gsub(/SENDER_LAST_NAME/, email_recipient.last_name).
                 gsub(/PACKAGE_NAME/, delivery.title).
                 gsub(/RECIPIENTS/, recipients_csv).
                 gsub(/PACKAGE_FROM/, Notifier.package_from(email_recipient))
                 )
    #self.cc(email_recipient.full_email_cc_list_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"
    _body_vars = {
      :sender_name => email_recipient.full_name_untruncated,
      :sender_first_name => email_recipient.first_name,
      :sender_last_name => email_recipient.last_name,
      :package_name => delivery.title,
      :dest_home_server_name => home_server.name,
      :recipients => recipients_csv,
      :package_url => package_url,
      :package_size => delivery.e_package.e_package_total_size_bytes,
      :package_files => delivery.e_package.e_package_total_files,
      :package_note => delivery.metadata_and_note,
      :package_from => Notifier.package_from(email_recipient)
    }
    sub_alternate_address_vars self.subject, _body_vars
    _plain_message = render_message("relay_failed_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("relay_failed_notification.text.html.rhtml", _body_vars)

    # putting this down here so we don't clobber the recipients substitution variable!
    self.recipients(email_recipient.full_email_address_descriptor)

    process_parts(_plain_message, _html_message, relay_failed_template.include_alternate_address_content)
  end

  def relay_failed_admin_notification(relay)
    relay_failed_admin_template = MTemplate.find_by_name('relay_failed_admin')
    relay_failed_admin_template.generate_rhtml(false)
    delivery = relay.delivery
    home_server = relay.home_server
    recipients = delivery.recipients_for_home_server(home_server)
    recipients_csv = recipients.collect(&:addressable_name).join(", ")
    url_host = EConfiguration.get_full_main_server_address
    err_msg = relay.last_status
    err_time = relay.last_status_time.strftime("%H:%M:%S %Y/%m/%d")
    this_server = HomeServer.this_server
    relay_details_url = url_host + relay_details_admin_package_path(delivery)

    Notifier.configure_mailer()
    email_recipients = User.super_admins.collect{|u| u.email}.join(", ")

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(relay_failed_admin_template.subject.
                 gsub(/SRC_HOME_SERVER_HOSTNAME/, this_server.hostname).
                 gsub(/SRC_HOME_SERVER_NAME/, this_server.name).
                 gsub(/DEST_HOME_SERVER_HOSTNAME/, home_server.hostname).
                 gsub(/DEST_HOME_SERVER_NAME/, home_server.name).
                 gsub(/PACKAGE_FROM/, Notifier.package_from(delivery.sender))
                 )
    self.sent_on(Time.now)
    content_type "multipart/alternative"
    _body_vars = {
      :sender_name => delivery.sender.full_name_untruncated,
      :sender_first_name => delivery.sender.first_name,
      :sender_last_name => delivery.sender.last_name,
      :package_name => delivery.title,
      :src_home_server_hostname => this_server.hostname,
      :src_home_server_name => this_server.name,
      :dest_home_server_hostname => home_server.hostname,
      :dest_home_server_name => home_server.name,
      :recipients => recipients_csv,
      :relay_error => err_msg,
      :error_time => err_time,
      :relay_details_url => relay_details_url,
      :package_from => Notifier.package_from(delivery.sender)
    }
    sub_alternate_address_vars self.subject, _body_vars
    _plain_message = render_message("relay_failed_admin_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("relay_failed_admin_notification.text.html.rhtml", _body_vars)

    # putting this down here so we don't clobber the recipients substitution variable!
    self.recipients(email_recipients)

    process_parts(_plain_message, _html_message, relay_failed_admin_template.include_alternate_address_content)
  end

  def ad_sync_failed_notification(email_recipient, error_message)
    Notifier.configure_mailer()
    ad_sync_failed_template = MTemplate.find_by_name('ad_sync_failed')
    ad_sync_failed_template.generate_rhtml(false)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(ad_sync_failed_template.subject)
    self.recipients(email_recipient.full_email_address_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"
    _body_vars = { :ad_sync_error => error_message }
    sub_alternate_address_vars self.subject, _body_vars
    _plain_message = render_message("ad_sync_failed_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("ad_sync_failed_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, ad_sync_failed_template.include_alternate_address_content)
  end

  def sync_failed_notification(home_server, email_recipient, error_message)
    Notifier.configure_mailer()
    sync_failed_template = MTemplate.find_by_name('sync_failed')
    sync_failed_template.generate_rhtml(false)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(sync_failed_template.subject.
                 gsub(/HOME_SERVER_NAME/, home_server.name).
                 gsub(/HOME_SERVER_HOSTNAME/, home_server.hostname).
                 gsub(/SYNC_ERROR/, error_message).
                 gsub(/USER_NAME/, email_recipient.full_name_untruncated).
                 gsub(/USER_FIRST_NAME/, email_recipient.first_name).
                 gsub(/USER_LAST_NAME/, email_recipient.last_name))

    self.recipients(email_recipient.full_email_address_descriptor)
    #self.cc(email_recipient.full_email_cc_list_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"
    _body_vars = { :user_name => email_recipient.full_name_untruncated,
                   :user_first_name => email_recipient.first_name,
                   :user_last_name => email_recipient.last_name,
                   :home_server_name => home_server.name,
                   :home_server_hostname => home_server.hostname,
                   :sync_error => error_message }
    sub_alternate_address_vars self.subject, _body_vars
    _plain_message = render_message("sync_failed_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("sync_failed_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, sync_failed_template.include_alternate_address_content)
  end

  def all_receipt_cc_recipients(recipient, delivery)
    if delivery.notifiable_on_receipt.nil?
      ccs = recipient.received_notification_cc.split(/[,; \n]/).
          collect{ |a| a.strip }.
          reject{ |a| a.blank? } rescue []
      (delivery.cc_on_receipt_recipients + ccs).uniq
    else
      delivery.cc_on_receipt_recipients
    end
  end

  def all_cc_recipients(recipient, delivery, multiple=false)
    ccs = delivery.notifications.on_receipt.empty? ? delivery.sender.full_email_cc_list : []
    if !multiple
      ccs += recipient.received_notification_cc.split(/[,; \n]/).
          collect{ |a| a.strip }.
          reject{ |a| a.blank? } rescue []
    elsif recipient.is_a?(Array)
      recipient.each do |r|
        user = AuthorizableUser.find_by_name(r) || AuthorizableUser.find_by_email(r)
        user_ccs = user.received_notification_cc.split(/[,; \n]/).
            collect{ |a| a.strip }.
            reject{ |a| a.blank? } rescue []
        ccs += user_ccs
      end
    end
      (delivery.all_cc_recipients + ccs).uniq
  end

  def all_cc_recipients_email(recipient, delivery, multiple=false)
    cc_recipients = all_cc_recipients(recipient, delivery, multiple)

    delivery.lookup_recipient_emails(cc_recipients)
  end

  def receipt_cc_recipients(recipient, delivery)
    delivery.lookup_recipients(all_receipt_cc_recipients(recipient, delivery), true).map{ |r| r.email rescue nil }.compact
  end

  def package_cc_receipt_on_received(recipient, package, delivery, notification)
    _package = delivery.e_package
    Notifier.configure_mailer()
    _received_template = MTemplate.find_by_name('package_cc_receipt_on_received')

    # generate rhtml file if it doesn't exist
    _received_template.generate_rhtml(false)
    _sender = delivery.sender

    self.from(Notifier.package_received_from_header(_sender))
    self.reply_to(_sender.full_email_address_descriptor)

    # sub in Faspex variables
    _subject = _received_template.subject
    _subject.gsub!('CC_NAME', notification["name"])
    _subject.gsub!('CC_EMAIL', notification["email"])
    _subject.gsub!('USER_NAME', (recipient.external_email_user? ? recipient.email : recipient.full_name_untruncated))
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', delivery.e_package.e_package_uuid)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('USER_NAME', (recipient.external_email_user? ? recipient.email :  recipient.full_name_untruncated))
    _subject.gsub!('USER_FIRST_NAME', (recipient.external_email_user? ? recipient.email :  recipient.first_name))
    _subject.gsub!('USER_LAST_NAME', (recipient.external_email_user? ? '' :  recipient.last_name))
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(_package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', _package.e_package_total_files.to_s + " file" + (_package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', _package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.metadata_and_note)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(recipient, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(recipient, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))
    self.subject(_subject)

    self.recipients(notification["email"])
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
        :cc_name => notification["name"],
        :cc_email => notification["email"],
        :user_name => (recipient.external_email_user? ? recipient.email : recipient.full_name_untruncated),
        :package_name => delivery.title,
        :package_uuid => delivery.e_package.e_package_uuid,
        :sender_name => _sender.full_name_untruncated,
        :sender_first_name => _sender.first_name,
        :sender_last_name => _sender.last_name,
        :sender_email => _sender.email,
        :sender_login => _sender.name,
        :package_date => delivery.created_at,
        :package_size => _package.e_package_total_size_bytes,
        :package_files => _package.e_package_total_files,
        :package_file_list_first_10 => _package.file_list_first_10,
        :package_note => delivery.metadata_and_note,
        :all_public_recipients => delivery.public_recipients.join(', '),
        :all_public_recipients_email => delivery.public_recipients_email.join(', '),
        :all_cc_recipients => all_cc_recipients(recipient, delivery).join(', '),
        :all_cc_recipients_email => all_cc_recipients_email(recipient, delivery).join(', '),
        :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("package_cc_receipt_on_received_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("package_cc_receipt_on_received_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, _received_template.include_alternate_address_content)
  end

  def package_cc_receipt_on_sent(package, delivery, notification)
    _package = delivery.e_package
    Notifier.configure_mailer()
    _received_template = MTemplate.find_by_name('package_cc_receipt_on_sent')
    # generate rhtml file if it doesn't exist
    _received_template.generate_rhtml(false)
    _sender = delivery.sender
    self.from(Notifier.package_received_from_header(_sender))
    self.reply_to(_sender.full_email_address_descriptor)

    # sub in Faspex variables
    _subject = _received_template.subject
    _subject.gsub!('CC_NAME', notification["name"])
    _subject.gsub!('CC_EMAIL', notification["email"])
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', delivery.e_package.e_package_uuid)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(_package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', _package.e_package_total_files.to_s + " file" + (_package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', _package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.metadata_and_note)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(delivery.recipients, delivery, true).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(delivery.recipients, delivery, true).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))

    self.subject(_subject)
    self.recipients(notification["email"])
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
        :cc_name => notification["name"],
        :cc_email => notification["email"],
        :package_name => delivery.title,
        :package_uuid => delivery.e_package.e_package_uuid,
        :sender_name => _sender.full_name_untruncated,
        :sender_first_name => _sender.first_name,
        :sender_last_name => _sender.last_name,
        :sender_email => _sender.email,
        :sender_login => _sender.name,
        :package_date => delivery.created_at,
        :package_size => _package.e_package_total_size_bytes,
        :package_files => _package.e_package_total_files,
        :package_file_list_first_10 => _package.file_list_first_10,
        :package_note => delivery.metadata_and_note,
        :all_public_recipients => delivery.public_recipients.join(', '),
        :all_public_recipients_email => delivery.public_recipients_email.join(', '),
        :all_cc_recipients => all_cc_recipients(delivery.recipients, delivery, true).join(', '),
        :all_cc_recipients_email => all_cc_recipients_email(delivery.recipients, delivery, true).join(', '),
        :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("package_cc_receipt_on_sent_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("package_cc_receipt_on_sent_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, _received_template.include_alternate_address_content)
  end

  def package_received_notification(recipient, delivery, package_url)
    _package = delivery.e_package

    Notifier.configure_mailer()
    _received_template = MTemplate.find_by_name('package_received')
    # generate rhtml file if it doesn't exist
    _received_template.generate_rhtml(false)
    _sender = delivery.sender
    expiration_info = expiration_info_for(recipient, delivery)

    self.from(Notifier.package_received_from_header(_sender))
    self.reply_to(_sender.full_email_address_descriptor)
    self.headers['X-Aspera-Filepkg-Location'] = package_url

    # sub in Faspex variables
    _subject = _received_template.subject
    _subject.gsub!('MOBILE_PACKAGE_URL', package_url)
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', delivery.e_package.e_package_uuid)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('USER_NAME', (recipient.external_email_user? ? recipient.email :  recipient.full_name_untruncated))
    _subject.gsub!('USER_FIRST_NAME', (recipient.external_email_user? ? recipient.email :  recipient.first_name))
    _subject.gsub!('USER_LAST_NAME', (recipient.external_email_user? ? '' :  recipient.last_name))
    _subject.gsub!('PACKAGE_URL', package_url)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(_package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', _package.e_package_total_files.to_s + " file" + (_package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', _package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.metadata_and_note)
    _subject.gsub!('LINK_EXPIRATION_INFO', expiration_info)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(recipient, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(recipient, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))

    self.subject(_subject)

    self.recipients(recipient.full_email_address_descriptor)
    #self.cc(self.receipt_cc_recipients(recipient, delivery))
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :user_name => (recipient.external_email_user? ? recipient.email : recipient.full_name_untruncated),
      :user_first_name => (recipient.external_email_user? ? recipient.email : recipient.first_name),
      :user_last_name => (recipient.external_email_user? ? '' : recipient.last_name),
      :package_name => delivery.title,
      :package_uuid => delivery.e_package.e_package_uuid,
      :package_url => package_url,
      :sender_name => _sender.full_name_untruncated,
      :sender_first_name => _sender.first_name,
      :sender_last_name => _sender.last_name,
      :sender_email => _sender.email,
      :sender_login => _sender.name,
      :package_date => delivery.created_at,
      :package_size => _package.e_package_total_size_bytes,
      :package_files => _package.e_package_total_files,
      :package_file_list_first_10 => _package.file_list_first_10,
      :package_note => delivery.metadata_and_note,
      :link_expiration_info => expiration_info,
      :all_public_recipients => delivery.public_recipients.join(', '),
      :all_public_recipients_email => delivery.public_recipients_email.join(', '),
      :all_cc_recipients => all_cc_recipients(recipient, delivery).join(', '),
      :all_cc_recipients_email => all_cc_recipients_email(recipient, delivery).join(', '),
      :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("package_received_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("package_received_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, _received_template.include_alternate_address_content)
  end

  def package_validation_failed_sender_notification(delivery, recipient, _sender)
    package = delivery.e_package
    _package_url = EConfiguration.get_full_main_server_address + sent_path(delivery.id, local: _sender.force_local_login?)

    # Specify SAML config to use on log in
    _package_url = Delivery.add_saml_provider_id(_package_url, _sender.saml_configuration_id) if _sender.saml?

    Notifier.configure_mailer()
    validation_failed_sender_template = MTemplate.find_by_name('package_validation_failed_sender')

    # generate rhtml file if it doesn't exist
    validation_failed_sender_template.generate_rhtml(false)

    self.from(Notifier.package_received_from_header(_sender))
    self.reply_to(_sender.full_email_address_descriptor)

    # sub in Faspex variables
    _subject = validation_failed_sender_template.subject
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', delivery.e_package.e_package_uuid)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('USER_NAME', (recipient.external_email_user? ? recipient.email :  recipient.full_name_untruncated))
    _subject.gsub!('USER_FIRST_NAME', (recipient.external_email_user? ? recipient.email :  recipient.first_name))
    _subject.gsub!('USER_LAST_NAME', (recipient.external_email_user? ? '' :  recipient.last_name))
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.calculate_size("/")))
    _subject.gsub!('PACKAGE_FILES', package.calculate_file_count("/").to_s + " file" + (package.calculate_file_count("/")==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.metadata_and_note)
    _subject.gsub!('PACKAGE_URL', _package_url)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(recipient, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(recipient, delivery).join(', '))
    _subject.gsub!('VALIDATION_ERROR', package.failed_list_first_10)
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))
    self.subject(_subject)

    self.recipients(_sender.full_email_address_descriptor)

    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
        :user_name => (recipient.external_email_user? ? recipient.email : recipient.full_name_untruncated),
        :package_name => package.e_package_name,
        :package_uuid => package.e_package_uuid,
        :package_url => _package_url,
        :sender_name => _sender.full_name_untruncated,
        :sender_first_name => _sender.first_name,
        :sender_last_name => _sender.last_name,
        :sender_email => _sender.email,
        :sender_login => _sender.name,
        :package_date => package.created_at,
        :package_size => package.calculate_size("/"),
        :package_files => package.calculate_file_count("/"),
        :package_file_list_first_10 => package.file_list_first_10,
        :package_note => package.e_package_note,
        :all_public_recipients => delivery.public_recipients.join(', '),
        :all_public_recipients_email => delivery.public_recipients_email.join(', '),
        :validation_error => package.failed_list_first_10,
        :all_cc_recipients => all_cc_recipients(recipient, delivery).join(', '),
        :all_cc_recipients_email => all_cc_recipients_email(recipient, delivery).join(', '),
        :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("package_validation_failed_sender_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("package_validation_failed_sender_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, validation_failed_sender_template.include_alternate_address_content)
  end

  def package_validation_failed_recipient_notification(delivery, recipient, _sender)
    package = delivery.e_package

    #Check whether this is a notification for members of a workgroup
    if delivery.workgroup_deliveries.first
      _package_url = EConfiguration.get_full_main_server_address + workgroup_delivery_path(delivery.workgroup_deliveries.first.recipient.id, delivery.workgroup_deliveries.first, local: recipient.force_local_login?, account: Digest::MD5.hexdigest(recipient.name.downcase))

      # Specify SAML config to use on log in
      _package_url = Delivery.add_saml_provider_id(_package_url, recipient.saml_configuration_id) if recipient.saml?
    else
      _package_url = EConfiguration.get_full_main_server_address + received_path(delivery.user_deliveries.first.id, local: recipient.force_local_login?, account: Digest::MD5.hexdigest(recipient.name.downcase))
      _package_url = Delivery.add_saml_provider_id(_package_url, recipient.saml_configuration_id) if recipient.saml?
    end

    Notifier.configure_mailer()
    validation_failed_recipient_template = MTemplate.find_by_name('package_validation_failed_recipient')

    # generate rhtml file if it doesn't exist
    validation_failed_recipient_template.generate_rhtml(false)

    self.from(Notifier.package_received_from_header(_sender))
    self.reply_to(_sender.full_email_address_descriptor)

    # sub in Faspex variables
    _subject = validation_failed_recipient_template.subject
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', delivery.e_package.e_package_uuid)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('USER_NAME', (recipient.external_email_user? ? recipient.email :  recipient.full_name_untruncated))
    _subject.gsub!('USER_FIRST_NAME', (recipient.external_email_user? ? recipient.email :  recipient.first_name))
    _subject.gsub!('USER_LAST_NAME', (recipient.external_email_user? ? '' :  recipient.last_name))
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.calculate_size("/")))
    _subject.gsub!('PACKAGE_FILES', package.calculate_file_count("/").to_s + " file" + (package.calculate_file_count("/")==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.metadata_and_note)
    _subject.gsub!('PACKAGE_URL', _package_url)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(recipient, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(recipient, delivery).join(', '))
    _subject.gsub!('VALIDATION_ERROR', package.failed_list_first_10)
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))
    self.subject(_subject)

    self.recipients(recipient.full_email_address_descriptor)

    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
        :user_name => (recipient.external_email_user? ? recipient.email : recipient.full_name_untruncated),
        :package_name => package.e_package_name,
        :package_uuid => package.e_package_uuid,
        :package_url => _package_url,
        :sender_name => _sender.full_name_untruncated,
        :sender_first_name => _sender.first_name,
        :sender_last_name => _sender.last_name,
        :sender_email => _sender.email,
        :sender_login => _sender.name,
        :package_date => package.created_at,
        :package_size => package.calculate_size("/"),
        :package_files => package.calculate_file_count("/"),
        :package_file_list_first_10 => package.file_list_first_10,
        :package_note => package.e_package_note,
        :all_public_recipients => delivery.public_recipients.join(', '),
        :all_public_recipients_email => delivery.public_recipients_email.join(', '),
        :validation_error => package.failed_list_first_10,
        :all_cc_recipients => all_cc_recipients(recipient, delivery).join(', '),
        :all_cc_recipients_email => all_cc_recipients_email(recipient, delivery).join(', '),
        :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("package_validation_failed_recipient_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("package_validation_failed_recipient_notification.text.html.rhtml", _body_vars)
    process_parts(_plain_message, _html_message, validation_failed_recipient_template.include_alternate_address_content)
  end

  def sub_alternate_address_vars subj, body_vars
    addrs = EConfiguration.first.alternate_addresses
    addrs.each_with_index do |a, i|
      name = "ALTERNATE_ADDRESS_" + (i + 1).to_s
      subj.gsub!(name, a.address_name)
      a = {name.downcase.to_sym => a.address_name}
      body_vars.merge!(a)
    end
    subj.gsub!('PRIMARY_ADDRESS', EConfiguration.first.e_server_address)
    body_vars.merge!({:primary_address => EConfiguration.first.e_server_address})
  end

  def process_parts(plain_message, html_message, include_alt_addrs=true)
    plain_message.gsub!(/&lt;br&gt;/, '\n')
    html_message.gsub!(/&lt;br&gt;/, '<br>')

    e_conf = EConfiguration.first
    _alternate_address = e_conf.alternate_addresses.select { |addr| addr.include_in_emails? }
    if e_conf.e_server_enable_alternate_address? && _alternate_address.present? && include_alt_addrs
      _main_address = EConfiguration.get_main_server_address_with_port
      html_link = html_message.scan(/.*/).join.scan(/(<a.*?)(#{_main_address})(.*?>)(.*?)(<\/a>)/)
      html_message += add_html_alternate_addresses(_alternate_address, html_link)
      plain_link = plain_message.scan(/.*/).join.scan(/\((URL: )(.*?)(#{_main_address})(.*?)\)/)
      plain_message += add_plain_alternate(_alternate_address, plain_link)
    end

    part :content_type => "text/plain", :body => plain_message
    part :content_type => "text/html", :body => html_message
  end

  def add_html_alternate_addresses(addrs, links)
    if links.empty?
      return ''
    end
    conf = EConfiguration.first
    to_add = '<br><br>' +
             '<span>Use these alternate addresses if needed:</span><br><br>' +
             '<div style="font-family: verdana, sans-serif; font-size: smaller;">'
    addrs.each do |a|
      to_add += '<div style="background-color: #EBEBED; padding: 5px;">'
      to_add += '<p style="font-weight: bold;">' + a.address_name.to_s + '</p>'
      to_add += '<p>' + a.description.to_s + '</p>'
      to_add += '<ul style="padding-left:0;margin-left:0">'

      links.each do |link|
        alt_addr = conf.address_with_port(a.address_name)
        link_name = link[3]
        if (link_name == 'http://' + link[1] + (link[2])[0..-3]) || (link_name == 'https://' + link[1] + (link[2])[0..-3])
          link_name = alt_addr + (link[2])[0..-3]
        elsif link_name == 'here'
          link_name = 'Reset Password'
        end
        to_add += '<li style="display:inline;padding-left:0;margin-left:0;padding-right:5px">'+ link[0] + alt_addr + link[2] + link_name + link[4] + '</li>'
      end
      to_add += '</ul></div><br>'
    end
    to_add += "</div>"
  end

 def add_plain_alternate(addrs, links)
   if links.empty?
     return ''
   end
   conf = EConfiguration.first
   to_add = '\n\nUse these alternate addresses if needed:\n\n'
   addrs.each do |a|
     to_add += a.address_name.to_s + '\n'
     to_add += a.description.to_s + '\n'
     links.each do |link|
       to_add += link[1] + conf.address_with_port(a.address_name) + link[3] + ' '
     end
     to_add += '\n\n'
   end
   to_add
 end

  def reset_password_notification(user, token)
    Notifier.configure_mailer
    _reset_pwd_template = MTemplate.find_by_name('forgot_password')
    _reset_url = EConfiguration.get_full_main_server_address + edit_reset_password_path(token) + (['LocalUser', 'SelfRegisteredUser'].include?(user.type) ? '?local=true' : '')

    _subject = _reset_pwd_template.subject
    _subject.gsub!('LOGIN', user.name)
    _subject.gsub!('USER_NAME', user.full_name_untruncated)
    _subject.gsub!('USER_FIRST_NAME', user.first_name)
    _subject.gsub!('USER_LAST_NAME', user.last_name)
    _subject.gsub!('RESET_URL', _reset_url)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(_subject)
    self.recipients(user.full_email_address_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      user_name: user.full_name_untruncated,
      user_first_name: user.first_name,
      user_last_name: user.last_name,
      reset_url: _reset_url,
      login: user.name
    }
    sub_alternate_address_vars _subject, _body_vars
    plain_message = render_message "forgot_password_notification.text.plain.rhtml", _body_vars
    html_message  = render_message "forgot_password_notification.text.html.rhtml", _body_vars
    process_parts plain_message, html_message, _reset_pwd_template.include_alternate_address_content
  end


  def email_settings_test(recipient_email)
    Notifier.configure_mailer

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject("Email settings test")
    self.recipients(recipient_email)
    self.sent_on(Time.now)
    self.body("If you received this message, your email settings are configured correctly.")
  end

  def license_warning
    Notifier.configure_mailer

    email_recipients = User.super_admins.collect(&:email).join(", ")
    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)
    self.subject("Warning: Faspex license exceeded")
    self.recipients(email_recipients)
    self.sent_on(Time.now)
    self.content_type "multipart/alternative"

    message = render_message("login_counts/_grace_period_warning.erb", {})
    process_parts(message, message, false)
  end

  def lockout_notice
    Notifier.configure_mailer

    email_recipients = User.super_admins.collect(&:email).join(", ")
    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)
    self.subject("Warning: Faspex license grace period expired")
    self.recipients(email_recipients)
    self.sent_on(Time.now)
    self.content_type "multipart/alternative"

    message = render_message("login_counts/_grace_period_expired.erb", {})
    process_parts(message, message, false)
  end

  def account_requested ccs
    Notifier.configure_mailer
    account_registrations_url = EConfiguration.get_full_main_server_address + account_registrations_path
    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)
    self.subject("Faspex account requested")
    self.sent_on(Time.now)
    self.content_type "multipart/alternative"
    self.cc(ccs)
    plain_message = render_message "registrations/_account_requested.text.plain.erb", account_registrations_url: account_registrations_url
    html_message  = render_message "registrations/_account_requested.text.html.erb",  account_registrations_url: account_registrations_url
    process_parts(plain_message, html_message, false)
  end

  def account_approved user, token
    Notifier.configure_mailer
    server_address = user.home_server.server_web_address + new_login_path(local: user.force_local_login?)
    template = MTemplate.find_by_name('account_approved')
    subject = template.subject
    subject.gsub!('LOGIN', user.name)
    subject.gsub!('USER_NAME', user.full_name_untruncated)
    subject.gsub!('USER_FIRST_NAME', user.first_name)
    subject.gsub!('USER_LAST_NAME', user.last_name)
    subject.gsub!('SERVER_ADDRESS', server_address)

    reset_password_url = EConfiguration.get_full_main_server_address + edit_reset_password_path(token) + (['LocalUser', 'SelfRegisteredUser'].include?(user.type) ? '?local=true' : '')
    from_header = Notifier.faspex_from_header

    self.from(from_header)
    self.reply_to(from_header)
    self.recipients(user.email)
    self.subject(subject)
    self.sent_on(Time.now)
    self.content_type "multipart/alternative"

    body_vars = {
      :user_name => user.full_name_untruncated,
      :user_first_name => user.first_name,
      :user_last_name => user.last_name,
      :server_address => server_address,
      :login => user.name,
      :comments => user.description || '',
      reset_url: reset_password_url
    }
    sub_alternate_address_vars subject, body_vars
    plain_message = render_message "account_approved_notification.text.plain.rhtml", body_vars
    html_message  = render_message "account_approved_notification.text.html.rhtml",  body_vars

    process_parts(plain_message, html_message, template.include_alternate_address_content)
  end

  def account_denied user_data
    Notifier.configure_mailer

    server_address = user_data['server_web_address'] + new_login_path(local: user_data['force_local_login'])
    template = MTemplate.find_by_name('account_denied')
    subject = template.subject
    subject.gsub!('LOGIN', user_data['name'])
    subject.gsub!('USER_NAME', user_data['full_name'])
    subject.gsub!('USER_FIRST_NAME', user_data['first_name'])
    subject.gsub!('USER_LAST_NAME', user_data['last_name'])
    subject.gsub!('SERVER_ADDRESS', server_address)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)
    self.recipients(user_data['email'])
    self.subject(subject)
    self.sent_on(Time.now)
    self.content_type "multipart/alternative"

    body_vars = {
      :user_name => user_data['full_name'],
      :user_first_name => user_data['first_name'],
      :user_last_name => user_data['last_name'],
      :server_address => server_address,
      :login => user_data['name']
    }
    sub_alternate_address_vars subject, body_vars
    plain_message = render_message "account_denied_notification.text.plain.rhtml", body_vars
    html_message  = render_message "account_denied_notification.text.html.rhtml",  body_vars
    process_parts(plain_message, html_message, template.include_alternate_address_content)
  end


  def personal_invitation invitation
    Notifier.configure_mailer
    template = MTemplate.find_by_name('personal_invitation')
    # generate rhtml file if it doesn't exist
    template.generate_rhtml(false)

    external_submission_url = EConfiguration.get_full_main_server_address + new_external_submission_path(passcode: invitation.passcode)
    expiration_info = passcode_expiration_info_for invitation
    description = invitation.description || ""

    _subject = template.subject
    _subject.gsub!('EMAIL', invitation.invitee.email)
    _subject.gsub!('RECIPIENT_NAME', invitation.inviter.full_name_untruncated)
    _subject.gsub!('RECIPIENT_FIRST_NAME', invitation.inviter.first_name)
    _subject.gsub!('RECIPIENT_LAST_NAME', invitation.inviter.last_name)
    _subject.gsub!('SUBMISSION_URL', external_submission_url)
    _subject.gsub!('LINK_EXPIRATION_INFO', expiration_info)
    _subject.gsub!('DESCRIPTION', description)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(_subject)
    self.recipients(invitation.invitee.full_email_address_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    description.gsub!(/\r\n/, '<br>')

    _body_vars = {
      :email => invitation.invitee.email,
      :recipient_name => invitation.inviter.full_name_untruncated,
      :recipient_first_name => invitation.inviter.first_name,
      :recipient_last_name => invitation.inviter.last_name,
      :submission_url => external_submission_url,
      :link_expiration_info => expiration_info,
      :description => description
    }

    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("personal_invitation_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("personal_invitation_notification.text.html.rhtml", _body_vars)

    process_parts(_plain_message, _html_message, template.include_alternate_address_content)
  end

  def personal_submit_confirmation delivery
    Notifier.configure_mailer
    template = MTemplate.find_by_name('personal_submit_confirmation')
    # generate rhtml file if it doesn't exist
    template.generate_rhtml(false)
    external_status_url = EConfiguration.get_full_main_server_address + external_submission_status_path(package: delivery.dbox_extusr_sender_pkg_passcode)
    recipient = delivery.user_deliveries.first.recipient

    _subject = template.subject
    _subject.gsub!('SENDER_EMAIL', delivery.sender.email)
    _subject.gsub!('RECIPIENT_NAME', recipient.full_name_untruncated)
    _subject.gsub!('RECIPIENT_FIRST_NAME', recipient.first_name)
    _subject.gsub!('RECIPIENT_LAST_NAME', recipient.last_name)
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', delivery.e_package.e_package_uuid)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(delivery.e_package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', delivery.e_package.e_package_total_files.to_s + " file" + (delivery.e_package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', delivery.e_package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', delivery.note)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(_subject)
    self.recipients(delivery.sender.full_email_address_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :sender_email => delivery.sender.email,
      :recipient_name => recipient.full_name_untruncated,
      :recipient_first_name => recipient.first_name,
      :recipient_last_name => recipient.last_name,
      :status_url => external_status_url,
      :package_name => delivery.title,
      :package_uuid => delivery.e_package.e_package_uuid,
      :package_date => delivery.created_at,
      :package_size => delivery.e_package.e_package_total_size_bytes,
      :package_files => delivery.e_package.e_package_total_files,
      :package_file_list_first_10 => delivery.e_package.file_list_first_10,
      :package_note => delivery.note
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("personal_submit_confirmation_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("personal_submit_confirmation_notification.text.html.rhtml", _body_vars)

    process_parts(_plain_message, _html_message, template.include_alternate_address_content)
  end

  def create_token_for(user, new_user=false)
    ResetPasswordToken.generate(user, new_user).token
  end

  def welcome_notification(recipient)
    Notifier.configure_mailer
    _welcome_template = MTemplate.find_by_name('welcome')
    # generate rhtml file if it doesn't exist
    _welcome_template.generate_rhtml(false)

    server_address = EConfiguration.get_full_main_server_address + new_login_path(local: recipient.force_local_login?)
    _reset_url = EConfiguration.get_full_main_server_address + edit_reset_password_path(create_token_for(recipient, true))

    _subject = _welcome_template.subject
    _subject.gsub!('LOGIN', recipient.name)
    _subject.gsub!('USER_NAME', recipient.full_name_untruncated)
    _subject.gsub!('USER_FIRST_NAME', recipient.first_name)
    _subject.gsub!('USER_LAST_NAME', recipient.last_name)
    _subject.gsub!('SERVER_ADDRESS', server_address)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(_subject)
    self.recipients(recipient.full_email_address_descriptor)
    #self.cc(recipient.full_email_cc_list_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :user_name => recipient.full_name_untruncated,
      :user_first_name => recipient.first_name,
      :user_last_name => recipient.last_name,
      :server_address => server_address,
      :login => recipient.name,
      :reset_url => _reset_url,
      :comments => recipient.description || ''
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("welcome_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("welcome_notification.text.html.rhtml", _body_vars)

    process_parts(_plain_message, _html_message, _welcome_template.include_alternate_address_content)
  end

  def workgroup_file_relay_notification(delivery, package, workgroup, notified_name, dest_node, dest_dir, list_all, type)
    Notifier.configure_mailer

    case type
    when 'start'
      template_name = 'relay_started_cc'
      file_name = 'relay_started_cc_notification'
    when 'error'
      template_name = 'relay_error_cc'
      file_name = 'relay_error_cc_notification'
    else
      template_name = 'relay_finished_cc'
      file_name = 'relay_finished_cc_notification'
    end

    notified_user = User.find_by_name(notified_name)

    _received_template = MTemplate.find_by_name(template_name)
    # generate rhtml file if it doesn't exist
    _received_template.generate_rhtml(false)
    _sender = User.find(delivery.sender_id)

    faspex_from_header = Notifier.faspex_from_header
    from_header = (workgroup.wg_member_access == EWorkgroup::MUTUAL_ACCESS) ?
      Notifier.package_received_from_header(_sender) : faspex_from_header
    self.from(from_header)
    self.reply_to(faspex_from_header)

    # sub in Faspex variables
    _subject = _received_template.subject
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', package.e_package_uuid)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', package.e_package_total_files.to_s + " file" + (package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('WORKGROUP_NAME', workgroup.e_wg_name)
    _subject.gsub!('PACKAGE_NOTE', delivery.note)
    _subject.gsub!('CC_NAME', notified_user ? notified_user.full_name_untruncated : notified_name)
    _subject.gsub!('CC_EMAIL', notified_user ? notified_user.email : notified_name)
    _subject.gsub!('DESTINATION_NODE', dest_node)
    _subject.gsub!('DESTINATION_DIRECTORY', dest_dir)
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('PACKAGE_FILE_LIST_ALL', list_all)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(notified_user, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(notified_user, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))

    self.subject(_subject)

    self.recipients(notified_user ? notified_user.full_email_address_descriptor : notified_name)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :package_name => delivery.title,
      :package_uuid => package.e_package_uuid,
      :sender_name => _sender.full_name_untruncated,
      :sender_first_name => _sender.first_name,
      :sender_last_name => _sender.last_name,
      :sender_email => _sender.email,
      :sender_login => _sender.name,
      :package_date => delivery.created_at,
      :package_size => package.e_package_total_size_bytes,
      :package_files => package.e_package_total_files,
      :workgroup_name => workgroup.e_wg_name,
      :package_note => delivery.metadata_and_note,
      :cc_name => notified_user ? notified_user.full_name_untruncated : notified_name,
      :cc_email => notified_user ? notified_user.email : notified_name,
      :destination_node => dest_node.dup,
      :destination_directory => dest_dir.dup,
      :package_file_list_first_10 => package.file_list_first_10,
      :package_file_list_all => list_all,
      :all_public_recipients => delivery.public_recipients.join(', '),
      :all_public_recipients_email => delivery.public_recipients_email.join(', '),
      :all_cc_recipients => all_cc_recipients(notified_user, delivery).join(', '),
      :all_cc_recipients_email => all_cc_recipients_email(notified_user, delivery).join(', '),
      :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message(file_name + ".text.plain.rhtml", _body_vars)
    _body_vars[:destination_node].gsub!('.', '<span>.</span>')
    _body_vars[:destination_directory].gsub!('.', '<span>.</span>')
    _html_message = render_message(file_name + ".text.html.rhtml", _body_vars)
    _html_message.gsub!(/&lt;span&gt;/, '<span>')
    _html_message.gsub!(/&lt;\/span&gt;/, '</span>')

    process_parts(_plain_message, _html_message, _received_template.include_alternate_address_content)
  end

  def workgroup_package_notification(recipient, package, delivery, package_url, workgroup)
    Notifier.configure_mailer
    _received_template = MTemplate.find_by_name('workgroup_package')
    # generate rhtml file if it doesn't exist
    _received_template.generate_rhtml(false)
    _sender = User.find(delivery.sender_id)
    expiration_info = expiration_info_for(workgroup, delivery)

    faspex_from_header = Notifier.faspex_from_header
    from_header = (workgroup.wg_member_access == EWorkgroup::MUTUAL_ACCESS) ?
        Notifier.package_received_from_header(_sender) : faspex_from_header
    self.from(from_header)
    self.reply_to(faspex_from_header)

    # sub in Faspex variables
    _subject = _received_template.subject
    _subject.gsub!('MOBILE_PACKAGE_URL', package_url)
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', package.e_package_uuid)
    _subject.gsub!('SENDER_NAME', _sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', _sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', _sender.last_name)
    _subject.gsub!('SENDER_EMAIL', _sender.email)
    _subject.gsub!('SENDER_LOGIN', _sender.name)
    _subject.gsub!('PACKAGE_URL', package_url)
    _subject.gsub!('USER_NAME', recipient.full_name_untruncated)
    _subject.gsub!('USER_FIRST_NAME', recipient.first_name)
    _subject.gsub!('USER_LAST_NAME', recipient.last_name)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', package.e_package_total_files.to_s + " file" + (package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('WORKGROUP_NAME', workgroup.e_wg_name)
    _subject.gsub!('PACKAGE_NOTE', delivery.note) if delivery.note
    _subject.gsub!('LINK_EXPIRATION_INFO', expiration_info)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(recipient, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(recipient, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(_sender))

    self.subject(_subject)

    self.recipients(recipient.full_email_address_descriptor)
    # self.cc(self.receipt_cc_recipients(recipient, delivery))
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :user_name => recipient.full_name_untruncated,
      :user_first_name => recipient.first_name,
      :user_last_name => recipient.last_name,
      :package_name => delivery.title,
      :package_uuid => package.e_package_uuid,
      :package_url => package_url,
      :sender_name => _sender.full_name_untruncated,
      :sender_first_name => _sender.first_name,
      :sender_last_name => _sender.last_name,
      :sender_email => _sender.email,
      :sender_login => _sender.name,
      :package_date => delivery.created_at,
      :package_size => package.e_package_total_size_bytes,
      :package_files => package.e_package_total_files,
      :package_file_list_first_10 => package.file_list_first_10,
      :package_note => delivery.metadata_and_note,
      :link_expiration_info => expiration_info,
      :workgroup_name => workgroup.e_wg_name,
      :all_public_recipients => delivery.public_recipients.join(', '),
      :all_public_recipients_email => delivery.public_recipients_email.join(', '),
      :all_cc_recipients => all_cc_recipients(recipient, delivery).join(', '),
      :all_cc_recipients_email => all_cc_recipients_email(recipient, delivery).join(', '),
      :package_from => Notifier.package_from(_sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("workgroup_package_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("workgroup_package_notification.text.html.rhtml", _body_vars)

    process_parts(_plain_message, _html_message, _received_template.include_alternate_address_content)
  end

  def dropbox_invitation membership_or_invitation
    Notifier.configure_mailer
    template = MTemplate.find_by_name 'dropbox_invitation'
    # generate rhtml file if it doesn't exist
    template.generate_rhtml false

    _dropbox_url     = EConfiguration.get_full_main_server_address + new_external_dropbox_submission_path(passcode: membership_or_invitation.passcode)
    _expiration_info = passcode_expiration_info_for membership_or_invitation
    _dropbox_name    = membership_or_invitation.e_workgroup.addressable_name
    _email           = membership_or_invitation.user.email
    _recipients      = membership_or_invitation.user.full_email_address_descriptor
    _dropbox_description = membership_or_invitation.description
    _subject = template.subject
    _subject.gsub! 'EMAIL',                _email
    _subject.gsub! 'DROPBOX_NAME',         _dropbox_name
    _subject.gsub! 'DROPBOX_URL',          _dropbox_url
    _subject.gsub! 'LINK_EXPIRATION_INFO', _expiration_info
    _subject.gsub! 'DESCRIPTION', _dropbox_description if _dropbox_description

    from_header = Notifier.faspex_from_header
    self.from     from_header
    self.reply_to from_header

    self.subject    _subject
    self.recipients _recipients
    self.sent_on    Time.now
    content_type    "multipart/alternative"

    _dropbox_description.gsub!(/\r\n/, '<br>')

    _body_vars = {
      email:                _email,
      dropbox_name:         _dropbox_name,
      dropbox_url:          _dropbox_url,
      link_expiration_info: _expiration_info,
      description:          _dropbox_description
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message 'dropbox_invitation_notification.text.plain.rhtml', _body_vars
    _html_message  = render_message 'dropbox_invitation_notification.text.html.rhtml',  _body_vars

    process_parts _plain_message, _html_message, template.include_alternate_address_content
  end

  def dropbox_submit_confirmation(wg_delivery, external_status_url)
    Notifier.configure_mailer
    template = MTemplate.find_by_name('dropbox_submit_confirmation')
    # generate rhtml file if it doesn't exist
    template.generate_rhtml(false)
    delivery_sender = wg_delivery.sender

    _subject = template.subject
    _subject.gsub!('SENDER_EMAIL', delivery_sender.email)
    _subject.gsub!('DROPBOX_NAME', wg_delivery.recipient.addressable_name)
    _subject.gsub!('PACKAGE_NAME', wg_delivery.title)
    _subject.gsub!('PACKAGE_UUID', wg_delivery.e_package.e_package_uuid)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(wg_delivery.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(wg_delivery.e_package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', wg_delivery.e_package.e_package_total_files.to_s + " file" + (wg_delivery.e_package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', wg_delivery.e_package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', wg_delivery.note)

    from_header = Notifier.faspex_from_header
    self.from(from_header)
    self.reply_to(from_header)

    self.subject(_subject)
    self.recipients(delivery_sender.full_email_address_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :sender_email => delivery_sender.email,
      :dropbox_name => wg_delivery.recipient.addressable_name,
      :status_url => external_status_url,
      :package_name => wg_delivery.title,
      :package_uuid => wg_delivery.e_package.e_package_uuid,
      :package_date => wg_delivery.created_at,
      :package_size => wg_delivery.e_package.e_package_total_size_bytes,
      :package_files => wg_delivery.e_package.e_package_total_files,
      :package_file_list_first_10 => wg_delivery.e_package.file_list_first_10,
      :package_note => wg_delivery.note
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("dropbox_submit_confirmation_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("dropbox_submit_confirmation_notification.text.html.rhtml", _body_vars)

    process_parts(_plain_message, _html_message, template.include_alternate_address_content)
  end

  def upload_result(delivery, package, sender, upload_result, status_url)
    Notifier.configure_mailer
    template = MTemplate.find_by_name('upload_result')
    # generate rhtml file if it doesn't exist
    template.generate_rhtml(false)

    from_header = Notifier.faspex_from_header
    self.from(from_header)

    _subject = template.subject
    _subject.gsub!('SENDER_NAME', sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', sender.last_name)
    _subject.gsub!('SENDER_EMAIL', sender.email)
    _subject.gsub!('SENDER_LOGIN', sender.name)
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', package.e_package_uuid)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(package.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', package.e_package_total_files.to_s + " file" + (package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', package.e_package_note)
    _subject.gsub!('UPLOAD_RESULT', upload_result)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(sender, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(sender, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(sender))

    self.reply_to(from_header)

    self.subject(_subject)
    self.recipients(sender.full_email_address_descriptor)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :sender_name => sender.full_name_untruncated,
      :sender_first_name => sender.first_name,
      :sender_last_name => sender.last_name,
      :sender_email => sender.email,
      :sender_login => sender.name,
      :status_url => status_url,
      :package_name => delivery.title,
      :package_uuid => package.e_package_uuid,
      :package_date => package.created_at,
      :package_size => package.e_package_total_size_bytes,
      :package_files => package.e_package_total_files,
      :package_file_list_first_10 => package.file_list_first_10,
      :package_note => package.e_package_note,
      :upload_result => upload_result,
      :all_public_recipients => delivery.public_recipients.join(', '),
      :all_public_recipients_email => delivery.public_recipients_email.join(', '),
      :all_cc_recipients => all_cc_recipients(sender, delivery).join(', '),
      :all_cc_recipients_email => all_cc_recipients_email(sender, delivery).join(', '),
      :package_from => Notifier.package_from(sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("upload_result_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("upload_result_notification.text.html.rhtml", _body_vars)

    process_parts(_plain_message, _html_message, template.include_alternate_address_content)
  end

  def upload_result_cc delivery, package, notification, upload_result
    Notifier.configure_mailer
    template = MTemplate.find_by_name('upload_result_cc')
    # generate rhtml file if it doesn't exist
    template.generate_rhtml(false)

    from_header = Notifier.faspex_from_header
    self.from(from_header)

    _subject = template.subject
    _subject.gsub!('CC_NAME', notification.name)
    _subject.gsub!('CC_EMAIL', notification.email)
    _subject.gsub!('SENDER_NAME', delivery.sender.full_name_untruncated)
    _subject.gsub!('SENDER_FIRST_NAME', delivery.sender.first_name)
    _subject.gsub!('SENDER_LAST_NAME', delivery.sender.last_name)
    _subject.gsub!('SENDER_EMAIL', delivery.sender.email)
    _subject.gsub!('SENDER_LOGIN', delivery.sender.name)
    _subject.gsub!('PACKAGE_NAME', delivery.title)
    _subject.gsub!('PACKAGE_UUID', package.e_package_uuid)
    _subject.gsub!('PACKAGE_DATE', Utils.admin_time_to_human_one_line(package.created_at))
    _subject.gsub!('PACKAGE_SIZE', @@helper.number_to_human_size(package.e_package_total_size_bytes))
    _subject.gsub!('PACKAGE_FILES', package.e_package_total_files.to_s + " file" + (package.e_package_total_files==1 ? '': 's'))
    _subject.gsub!('PACKAGE_FILE_LIST_FIRST_10', package.file_list_first_10)
    _subject.gsub!('PACKAGE_NOTE', package.e_package_note)
    _subject.gsub!('UPLOAD_RESULT', upload_result)
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS_EMAIL', delivery.public_recipients_email.join(', '))
    _subject.gsub!('ALL_PUBLIC_RECIPIENTS', delivery.public_recipients.join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS_EMAIL', all_cc_recipients_email(delivery.sender, delivery).join(', '))
    _subject.gsub!('ALL_CC_RECIPIENTS', all_cc_recipients(delivery.sender, delivery).join(', '))
    _subject.gsub!('PACKAGE_FROM', Notifier.package_from(delivery.sender))


    self.reply_to(from_header)

    self.subject(_subject)
    self.recipients(notification.email)
    self.sent_on(Time.now)
    content_type "multipart/alternative"

    _body_vars = {
      :cc_name => notification.name,
      :cc_email => notification.email,
      :sender_name => delivery.sender.full_name_untruncated,
      :sender_first_name => delivery.sender.first_name,
      :sender_last_name => delivery.sender.last_name,
      :sender_email => delivery.sender.email,
      :sender_login => delivery.sender.name,
      :package_name => delivery.title,
      :package_uuid => package.e_package_uuid,
      :package_date => package.created_at,
      :package_size => package.e_package_total_size_bytes,
      :package_files => package.e_package_total_files,
      :package_file_list_first_10 => package.file_list_first_10,
      :package_note => package.e_package_note,
      :upload_result => upload_result,
      :all_public_recipients => delivery.public_recipients.join(', '),
      :all_public_recipients_email => delivery.public_recipients_email.join(', '),
      :all_cc_recipients => all_cc_recipients(delivery.sender, delivery).join(', '),
      :all_cc_recipients_email => all_cc_recipients_email(delivery.sender, delivery).join(', '),
      :package_from => Notifier.package_from(delivery.sender)
    }
    sub_alternate_address_vars _subject, _body_vars
    _plain_message = render_message("upload_result_cc_notification.text.plain.rhtml", _body_vars)
    _html_message = render_message("upload_result_cc_notification.text.html.rhtml", _body_vars)

    process_parts(_plain_message, _html_message, template.include_alternate_address_content)
  end

  private

  def expiration_info_for(recipient, delivery_send)
    personal_or_workgroup_delivery = delivery_send.recipient_deliveries.find_by_recipient_id(recipient.id)
    expires_at = personal_or_workgroup_delivery.link_expires_at
    expires_at ? "The download link will expire #{Utils.admin_time_to_human_one_line(expires_at)} #{Time.now.zone}" : ""
  end

  def passcode_expiration_info_for membership_or_invitation
    case membership_or_invitation.expiration
    when 'after_one_upload'
      info = 'after one upload'
    when 'after_one_upload_or_date'
      info = 'after one upload or ' + Utils.admin_time_to_human_one_line(membership_or_invitation.expires_at) + " " + Time.now.zone
    when 'on_date'
      info = 'on ' + Utils.admin_time_to_human_one_line(membership_or_invitation.expires_at) + " " + Time.now.zone
    else
      info = ''
    end
    info.present? ? "The submission link will expire #{info}." : ""
  end
end
